@@ -7,8 +7,10 @@ from .composition import *
 from .pathway import *
 from .compositionfunctionapproximator import *
 from .showgraph import *
+from .report import *
 
 __all__ = list(composition.__all__)
 __all__.extend(pathway.__all__)
 __all__.extend(compositionfunctionapproximator.__all__)
 __all__.extend(showgraph.__all__)
+__all__.extend(report.__all__)
