@@ -1078,7 +1078,7 @@ class Log:
             delete the data, but leave the entry itself (if `False`).
 
             .. note::
-                This option is included for generality and potential future features, but is not advised;
+                This option is included for generality and potential future state_features, but is not advised;
                 the Log interface (e.g., the `logged_items <Log.logged_items>` interface generally assumes that
                 the only `entries <Log.entries>` in a log are ones with data.
 
@@ -1886,9 +1886,9 @@ def _log_trials_and_runs(composition, curr_condition: tc.enum(LogCondition.TRIAL
     for mech in composition.mechanisms:
         for component in mech.log.loggable_components:
             if component.logPref & curr_condition:
-                # value = LogEntry((composition.scheduler.clock.time.run,
-                #                   composition.scheduler.clock.time.trial,
-                #                   composition.scheduler.clock.time.time_step),
+                # value = LogEntry((composition.scheduler.get_clock(context).time.run,
+                #                   composition.scheduler.get_clock(context).time.trial,
+                #                   composition.scheduler.get_clock(context).time.time_step),
                 #                  # context,
                 #                  curr_condition,
                 #                  component.value)
@@ -1898,9 +1898,9 @@ def _log_trials_and_runs(composition, curr_condition: tc.enum(LogCondition.TRIAL
         for proj in mech.afferents:
             for component in proj.log.loggable_components:
                 if component.logPref & curr_condition:
-                    # value = LogEntry((composition.scheduler.clock.time.run,
-                    #                   composition.scheduler.clock.time.trial,
-                    #                   composition.scheduler.clock.time.time_step),
+                    # value = LogEntry((composition.scheduler.get_clock(context).time.run,
+                    #                   composition.scheduler.get_clock(context).time.trial,
+                    #                   composition.scheduler.get_clock(context).time.time_step),
                     #                  context,
                     #                  component.value)
                     # component.log._log_value(value, context)
@@ -1910,9 +1910,9 @@ def _log_trials_and_runs(composition, curr_condition: tc.enum(LogCondition.TRIAL
     # for proj in composition.projections:
     #     for component in proj.log.loggable_components:
     #         if component.logPref & curr_condition:
-    #             value = LogEntry((composition.scheduler.clock.time.run,
-    #                               composition.scheduler.clock.time.trial,
-    #                               composition.scheduler.clock.time.time_step),
+    #             value = LogEntry((composition.scheduler.get_clock(context).time.run,
+    #                               composition.scheduler.get_clock(context).time.trial,
+    #                               composition.scheduler.get_clock(context).time.time_step),
     #                              context,
     #                              component.value)
     #             component.log._log_value(value, context)
