@@ -78,7 +78,7 @@ different execution contexts and can be modified by modulated `ModulatorySignal_
     <Parameter.set>` method, see `BasicsAndPrimer_Parameters`.
 
 
-.. _technical_note::
+.. technical_note::
 
     Developers must keep in mind state when writing new Components for PsyNeuLink. Any parameters or values that may
     change during a `run <Run_Overview>` must become stateful Parameters, or they are at risk of computational
@@ -177,6 +177,23 @@ To create new Parameters, reference this example of a new class *B*
         ControlSignal's function is always a `TransferWithCosts` and is
         not user-specifiable.
 
+.. technical_note::
+    _user_specified
+    ---------------
+        Currently, default argument values for __init__ methods of
+        Components are all None to allow distinguishing between a
+        default value a user actually passed in during construction and
+        a default automatically assigned. Consider
+
+            1. `pnl.TransferMechanism(noise=[[0, 0]])`
+            2. `pnl.TransferMechanism(default_variable=[[0]], noise=[[0, 0]])`
+
+        In case 1, the default variable is not specified by the user,
+        and is `flexible <DefaultsFlexibility.FLEXIBLE>` as a result.
+        Since noise indicates a shape, `[[0, 0]]` is implicitly assigned
+        as the Mechanism's default variable. This is intended as a
+        convenience specification. In case 2, the variable and noise are
+        incompatible, and an error is raised.
 
 Using Parameters
 ^^^^^^^^^^^^^^^^
@@ -207,8 +224,8 @@ You should avoid using `dot notation <Parameter_Dot_Notation>` in internal code,
 |                  |               |based on execution context                  |                                         |
 +------------------+---------------+--------------------------------------------+-----------------------------------------+
 |    modulable     |     False     |if True, the parameter can be modulated     |Currently this does not determine what   |
-|                  |               |(if it belongs to a Mechanism or Projection |gets a ParameterPort, but in the future |
-|                  |               | it is assigned a `ParameterPort`)         |it should                                |
+|                  |               |(if it belongs to a Mechanism or Projection |gets a ParameterPort, but in the future  |
+|                  |               | it is assigned a `ParameterPort`)          |it should                                |
 +------------------+---------------+--------------------------------------------+-----------------------------------------+
 |    read_only     |     False     |whether the user should be able to set the  |Can be manually set, but will trigger a  |
 |                  |               |value or not (e.g. variable and value are   |warning unless override=True             |
@@ -227,7 +244,7 @@ You should avoid using `dot notation <Parameter_Dot_Notation>` in internal code,
 +------------------+---------------+--------------------------------------------+-----------------------------------------+
 |      getter      |     None      |hook that allows overriding the retrieval of|kwargs self, owning_component, and       |
 |                  |               |values based on a supplied method           |context will be passed in if your        |
-|                  |               |(e.g. _output_port_variable_getter)        |method uses them. self - the Parameter   |
+|                  |               |(e.g. _output_port_variable_getter)        |method uses them. self - the Parameter    |
 |                  |               |                                            |calling the setter; owning_component -   |
 |                  |               |                                            |the Component to which the Parameter     |
 |                  |               |                                            |belongs; context - the context           |
@@ -837,6 +854,8 @@ class Parameter(ParameterBase):
         # attributes will be taken from
         _inherited_source=None,
         _user_specified=False,
+        # if modulated, set to the ParameterPort
+        _port=None,
         **kwargs
     ):
         if isinstance(aliases, str):
@@ -894,6 +913,7 @@ class Parameter(ParameterBase):
             _inherited=_inherited,
             _inherited_source=_inherited_source,
             _user_specified=_user_specified,
+            _port=_port,
             **kwargs
         )
 
@@ -1267,7 +1287,8 @@ class Parameter(ParameterBase):
         value = self._set(self._parse(value), context, skip_history, skip_log, **kwargs)
 
         try:
-            value = value.__self__
+            if isinstance(value.__self__, Component):
+                value = value.__self__
         except AttributeError:
             pass
 
