@@ -20,6 +20,8 @@
 CONTENTS
 --------
 
+* `deprecation_warning`
+
 *TYPE CHECKING VALUE COMPARISON*
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -121,7 +123,7 @@ __all__ = [
     'get_all_explicit_arguments', 'get_modulationOperation_name', 'get_value_from_array',
     'insert_list', 'is_matrix_spec', 'all_within_range',
     'is_comparison_operator',  'iscompatible', 'is_component', 'is_distance_metric', 'is_iterable', 'is_matrix',
-    'is_modulation_operation', 'is_numeric', 'is_numeric_or_none', 'is_same_function_spec', 'is_unit_interval',
+    'is_modulation_operation', 'is_numeric', 'is_numeric_or_none', 'is_number', 'is_same_function_spec', 'is_unit_interval',
     'is_value_spec',
     'kwCompatibilityLength', 'kwCompatibilityNumeric', 'kwCompatibilityType',
     'make_readonly_property',
@@ -146,6 +148,34 @@ class UtilitiesError(Exception):
         return repr(self.error_value)
 
 
+
+def deprecation_warning(component, kwargs:dict, deprecated_args:dict) -> dict:
+    """Identify and warn about any deprecated args, and return their values for reassignment
+    Format of deprecated_args dict:  {'deprecated_arg_name':'real_arg"name')}
+    Format of returned dict:  {'real_arg_name':<value assigned to deprecated arg>)}
+    """
+    value_assignments = dict()
+    for deprecated_arg in deprecated_args:
+        if deprecated_arg in kwargs:
+            real_arg = deprecated_args[deprecated_arg]
+            arg_value = kwargs.pop(deprecated_arg)
+            if arg_value:
+                # Value for real arg was also specified:
+                warnings.warn(f"Both '{deprecated_arg}' and '{real_arg}' "
+                              f"were specified in the constructor for a(n) {component.__class__.__name__}; "
+                              f"{deprecated_arg} ({arg_value}) will be used,"
+                              f"but note that it is deprecated  and may be removed in the future.")
+            else:
+                # Only deprecated arg was specified:
+                warnings.warn(f"'{deprecated_arg}' was specified in the constructor for a(n)"
+                              f" {component.__class__.__name__}; note that this has been deprecated "
+                              f"and may be removed in the future; '{real_arg}' "
+                              f"should be used instead.")
+            value_assignments.update({real_arg:arg_value})
+        continue
+    return value_assignments
+
+
 MODULATION_OVERRIDE = 'Modulation.OVERRIDE'
 MODULATION_MULTIPLY = 'Modulation.MULTIPLY'
 MODULATION_ADD = 'Modulation.ADD'
@@ -293,6 +323,13 @@ def is_numeric(x):
     return iscompatible(x, **{kwCompatibilityNumeric:True, kwCompatibilityLength:0})
 
 
+def is_number(x):
+    return (
+        isinstance(x, numbers.Number)
+        and not isinstance(x, (bool, Enum))
+    )
+
+
 def is_matrix_spec(m):
     return isinstance(m, str) and m in MATRIX_KEYWORD_VALUES
 
@@ -387,7 +424,6 @@ def iscompatible(candidate, reference=None, **kargs):
     :param args: (dict)
     :return:
     """
-
     # If the two are equal, can settle it right here
     # IMPLEMENTATION NOTE: remove the duck typing when numpy supports a direct comparison of iterables
     try:
@@ -458,8 +494,8 @@ def iscompatible(candidate, reference=None, **kargs):
     # MODIFIED 10/29/17 NEW:
     # IMPLEMENTATION NOTE: This allows a number in an ndarray to match a float or int
     # If both the candidate and reference are either a number or an ndarray of dim 0, consider it a match
-    if ((isinstance(candidate, numbers.Number) or (isinstance(candidate, np.ndarray) and candidate.ndim == 0)) or
-            (isinstance(reference, numbers.Number) or (isinstance(reference, np.ndarray) and reference.ndim == 0))):
+    if ((is_number(candidate) or (isinstance(candidate, np.ndarray) and candidate.ndim == 0)) or
+            (is_number(reference) or (isinstance(reference, np.ndarray) and reference.ndim == 0))):
         return True
     # MODIFIED 10/29/17 END
 
@@ -468,14 +504,14 @@ def iscompatible(candidate, reference=None, **kargs):
     #   should be added as option in future (i.e., to disallow it)
     if (isinstance(candidate, match_type) or
             (isinstance(candidate, (list, np.ndarray)) and (issubclass(match_type, (list, np.ndarray)))) or
-            (isinstance(candidate, numbers.Number) and issubclass(match_type,numbers.Number)) or
+            (is_number(candidate) and issubclass(match_type,numbers.Number)) or
             # IMPLEMENTATION NOTE: Allow UserDict types to match dict (make this an option in the future)
             (isinstance(candidate, UserDict) and match_type is dict) or
             # IMPLEMENTATION NOTE: Allow UserList types to match list (make this an option in the future)
             (isinstance(candidate, UserList) and match_type is list) or
             # IMPLEMENTATION NOTE: This is needed when kwCompatiblityType is not specified
             #                      and so match_type==list as default
-            (isinstance(candidate, numbers.Number) and issubclass(match_type,list)) or
+            (is_number(candidate) and issubclass(match_type,list)) or
                 (isinstance(candidate, np.ndarray) and issubclass(match_type,list))
         ):
 
@@ -493,12 +529,10 @@ def iscompatible(candidate, reference=None, **kargs):
         #                 candidate.value is match_type.__dict__['_member_map_'][candidate.name].value):
         #         return False
         # This version simply enforces the constraint that, if either is an enum of some sort, then both must be
-        if (kargs[kwCompatibilityType] is Enum and
-                (isinstance(candidate, Enum) != isinstance(match_type, (Enum, IntEnum, EnumMeta)))
-            ):
-            return False
+        if kargs[kwCompatibilityType] is Enum:
+            return isinstance(candidate, Enum) == isinstance(match_type, (Enum, IntEnum, EnumMeta))
 
-        if isinstance(candidate, numbers.Number):
+        if is_number(candidate):
             return True
         if number_only:
             if isinstance(candidate, np.ndarray) and candidate.ndim ==0 and np.isreal(candidate):
@@ -516,7 +550,7 @@ def iscompatible(candidate, reference=None, **kargs):
                         else:
                             return True
                 else:
-                    if not isinstance(value, numbers.Number):
+                    if not is_number(value):
                         try:
                             # True for autograd ArrayBox (and maybe other types?)
                             # if isinstance(value._value, numbers.Number):
@@ -544,6 +578,9 @@ def iscompatible(candidate, reference=None, **kargs):
                         candidate = np.asarray(candidate)
                     if isinstance(reference, np.matrix):
                         reference = np.asarray(reference)
+                    if (isinstance(candidate, np.ndarray) and isinstance(reference, np.ndarray)
+                            and candidate.shape == reference.shape):
+                        return True
                     cr = zip(candidate, reference)
                     if all(iscompatible(c, r, **kargs) for c, r in cr):
                         return True
@@ -736,11 +773,15 @@ def copy_iterable_with_shared(obj, shared_types=None, memo=None):
     all_types_using_recursion = dict_types + list_types + tuple_types
 
     if isinstance(obj, dict_types):
-        result = obj.__class__()
+        result = copy.copy(obj)
+        del_keys = set()
         for (k, v) in obj.items():
             # key can never be unhashable dict or list
             new_k = k if isinstance(k, shared_types) else copy.deepcopy(k, memo)
 
+            if new_k is not k:
+                del_keys.add(k)
+
             if isinstance(v, all_types_using_recursion):
                 new_v = copy_iterable_with_shared(v, shared_types, memo)
             elif isinstance(v, shared_types):
@@ -753,6 +794,8 @@ def copy_iterable_with_shared(obj, shared_types=None, memo=None):
             except UtilitiesError:
                 # handle ReadOnlyOrderedDict
                 result.__additem__(new_k, new_v)
+        for k in del_keys:
+            del result[k]
 
     elif isinstance(obj, list_types + tuple_types):
         is_tuple = isinstance(obj, tuple_types)
@@ -1441,8 +1484,8 @@ def convert_all_elements_to_np_array(arr, cast_from=None, cast_to=None):
         -------
         a numpy array containing the converted **arr**
     """
-    if isinstance(arr, np.ndarray) and arr.ndim == 0:
-        if cast_from is not None and isinstance(arr.item(0), cast_from):
+    if isinstance(arr, np.ndarray) and arr.dtype != object:
+        if cast_from is not None and arr.dtype == cast_from:
             return np.asarray(arr, dtype=cast_to)
         else:
             return arr
@@ -1536,7 +1579,16 @@ def safe_equals(x, y):
             else:
                 raise ValueError
         except (ValueError, DeprecationWarning, FutureWarning):
-            return np.array_equal(x, y)
+            try:
+                return np.array_equal(x, y)
+            except DeprecationWarning:
+                len_x = len(x)
+                return (
+                    len_x == len(y)
+                    and all([
+                        safe_equals(x[i], y[i]) for i in range(len_x)
+                    ])
+                )
 
 
 @tc.typecheck
