@@ -10,7 +10,6 @@
 import ast
 import warnings
 import numpy as np
-from functools import reduce
 
 from llvmlite import ir
 from contextlib import contextmanager
@@ -23,99 +22,46 @@ from . import helpers
 from .debug import debug_env
 
 class UserDefinedFunctionVisitor(ast.NodeVisitor):
-    def __init__(self, ctx, builder, func_globals, func_params, arg_in, arg_out):
+    def __init__(self, ctx, var_builder, builder, func_globals, func_params, arg_in, arg_out):
         self.ctx = ctx
         self.builder = builder
+        self.var_builder = var_builder
         self.func_params = func_params
         self.arg_in = arg_in
         self.arg_out = arg_out
 
         #setup default functions
-        def _list_sum(x):
-            # HACK: Obtain polymorphic addition function by visiting the add node
-            # this should ideally be moved to an explicit helper
-            add_func = self.visit_Add(None)
-
-            total_sum = builder.alloca(x.type.pointee.element)
-            builder.store(total_sum.type.pointee(None), total_sum)
-            with helpers.array_ptr_loop(builder, x, "list_sum") as (b, idx):
-                curr_val = b.gep(x, [ctx.int32_ty(0), idx])
-                tmp = add_func(total_sum, curr_val)
-                if helpers.is_pointer(tmp):
-                    tmp = builder.load(tmp)
-                b.store(tmp, total_sum)
-            return total_sum
-
-        def _len(x):
-            x_ty = x.type
-            if helpers.is_pointer(x):
-                x_ty = x_ty.pointee
-            return ctx.float_ty(len(x_ty))
-
-        def _tanh(x):
-            output_ptr = builder.alloca(x.type.pointee)
-            helpers.call_elementwise_operation(self.ctx, self.builder, x, helpers.tanh, output_ptr)
-            return output_ptr
-
-        def _exp(x):
-            output_ptr = builder.alloca(x.type.pointee)
-            helpers.call_elementwise_operation(self.ctx, self.builder, x, helpers.exp, output_ptr)
-            return output_ptr
-
-        # numpy's max function differs greatly from that of python's buiiltin max
-        # see: https://numpy.org/doc/stable/reference/generated/numpy.amax.html#numpy.amax
-        def _max_numpy(x):
-            assert helpers.is_vector(x) or helpers.is_2d_matrix(x), "Attempted to call max on invalid variable! Only 1-d and 2-d lists are supported!"
-            curr = builder.alloca(ctx.float_ty)
-            builder.store(ctx.float_ty('NaN'), curr)
-            for (element_ptr,) in helpers.recursive_iterate_arrays(ctx, builder, x):
-                element = builder.load(element_ptr)
-                greater = builder.fcmp_unordered('>', element, builder.load(curr))
-                with builder.if_then(greater):
-                    builder.store(element, curr)
-            return curr
-
-        # see: https://docs.python.org/3/library/functions.html#max
-        def _max(*args):
-            if len(args) == 1 and helpers.is_vector(args[0]):
-                curr = builder.alloca(ctx.float_ty)
-                builder.store(ctx.float_ty('NaN'), curr)
-                for (element_ptr,) in helpers.recursive_iterate_arrays(ctx, builder, args[0]):
-                    element = builder.load(element_ptr)
-                    greater = builder.fcmp_unordered('>', element, builder.load(curr))
-                    with builder.if_then(greater):
-                        builder.store(element, curr)
-                return curr
-            elif len(args) > 1 and all(a.type == args[0].type for a in args):
-                curr = builder.alloca(ctx.float_ty)
-                builder.store(ctx.float_ty('NaN'), curr)
-                for element in args:
-                    if helpers.is_pointer(element):
-                        element = builder.load(element)
-                    greater = builder.fcmp_unordered('>', element, builder.load(curr))
-                    with builder.if_then(greater):
-                        builder.store(element, curr)
-                return curr
-            assert False, "Attempted to call max with invalid arguments!"
         self.register = {
-            "sum": _list_sum,
-            "len": _len,
+            "sum": self.call_builtin_horizontal_sum,
+            "len": self.call_builtin_len,
             "float": ctx.float_ty,
             "int": ctx.int32_ty,
-            "max": _max,
+            "max": self.call_builtin_max,
         }
 
+        # Numpy function calls
+        def get_np_cmp(cmp_op):
+            def np_cmp(builder, x, y):
+                op = self._generate_fcmp_handler(None, None, cmp_op)
+                if helpers.is_pointer(x):
+                    x = builder.load(x)
+                if helpers.is_pointer(y):
+                    y = builder.load(y)
+                return self._do_bin_op(builder, x, y, op)
+
+            return np_cmp
+
         # setup numpy
         numpy_handlers = {
-            'tanh': _tanh,
-            'exp': _exp,
-            'equal': self._generate_fcmp_handler(self.ctx, self.builder, "=="),
-            'not_equal': self._generate_fcmp_handler(self.ctx, self.builder, "!="),
-            'less': self._generate_fcmp_handler(self.ctx, self.builder, "<"),
-            'less_equal': self._generate_fcmp_handler(self.ctx, self.builder, "<="),
-            'greater': self._generate_fcmp_handler(self.ctx, self.builder, ">"),
-            'greater_equal': self._generate_fcmp_handler(self.ctx, self.builder, ">="),
-            "max": _max_numpy,
+            'tanh': self.call_builtin_np_tanh,
+            'exp': self.call_builtin_np_exp,
+            'equal': get_np_cmp("=="),
+            'not_equal': get_np_cmp("!="),
+            'less': get_np_cmp("<"),
+            'less_equal': get_np_cmp("<="),
+            'greater': get_np_cmp(">"),
+            'greater_equal': get_np_cmp(">="),
+            "max": self.call_builtin_np_max,
         }
 
         for k, v in func_globals.items():
@@ -129,6 +75,14 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
         self.name_constants = name_constants
         super().__init__()
 
+    def get_rval(self, val):
+        if helpers.is_pointer(val):
+            return self.builder.load(val)
+        return val
+
+    def is_lval(self, val):
+        return helpers.is_pointer(val)
+
     def visit_arguments(self, node):
         args = node.args
         variable = args[0]
@@ -145,229 +99,72 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
             else:
                 self.register[param.arg] = self.func_params[param.arg]
 
-    def _generate_binop(self, x, y, callback):
-        # unpack scalars from pointers
-        if helpers.is_floating_point(x) and helpers.is_pointer(x):
-            x = self.builder.load(x)
-        if helpers.is_floating_point(y) and helpers.is_pointer(y):
-            y = self.builder.load(y)
-
-        return callback(self.ctx, self.builder, x, y)
-
     def visit_Add(self, node):
-        def _add_vec(ctx, builder, u, v):
-            assert u.type == v.type
-
-            dim = len(u.type.pointee)
-            output_vec = builder.alloca(u.type.pointee)
-
-            # Get the pointer to the first element of the array to convert from [? x double]* -> double*
-            vec_u = builder.gep(u, [ctx.int32_ty(0), ctx.int32_ty(0)])
-            vec_v = builder.gep(v, [ctx.int32_ty(0), ctx.int32_ty(0)])
-            vec_out = builder.gep(output_vec, [ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_vec_add")
-            builder.call(builtin, [vec_u, vec_v, ctx.int32_ty(dim), vec_out])
-            return output_vec
-
-        def _add_mat(ctx, builder, m1, m2):
-            assert m1.type == m2.type
-
-            x = len(m1.type.pointee)
-            y = len(m1.type.pointee.element)
-
-            output_mat = builder.alloca(m1.type.pointee)
-
-            m1_ptr = builder.gep(m1, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-            m2_ptr = builder.gep(m2, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-            output_ptr = builder.gep(output_mat, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_mat_add")
-            builder.call(builtin, [m1_ptr, m2_ptr, ctx.int32_ty(x), ctx.int32_ty(y), output_ptr])
-            return output_mat
-
-        def _add_mat_scalar(ctx, builder, m1, s):
-            x = len(m1.type.pointee)
-            y = len(m1.type.pointee.element)
-            output_mat = builder.alloca(m1.type.pointee)
-
-            m1_ptr = builder.gep(m1, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-            output_ptr = builder.gep(output_mat, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_mat_scalar_add")
-            builder.call(builtin, [m1_ptr, s, ctx.int32_ty(x), ctx.int32_ty(y), output_ptr])
-            return output_mat
-
-        def _add_vec_scalar(ctx, builder, u, s):
-            output_vec = builder.alloca(u.type.pointee)
-
-            helpers.call_elementwise_operation(ctx, builder, u, lambda ctx, builder, x: builder.fadd(x, s), output_vec)
-            return output_vec
-
-        def _add(x, y):
-            if helpers.is_floating_point(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fadd(x,y))
-            elif helpers.is_vector(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _add_vec_scalar)
-            elif helpers.is_floating_point(x) and helpers.is_vector(y):
-                return self._generate_binop(y, x, _add_vec_scalar)
-            elif helpers.is_2d_matrix(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _add_mat_scalar)
-            elif helpers.is_floating_point(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(y, x, _add_mat_scalar)
-            elif helpers.is_vector(x) and helpers.is_vector(y):
-                return self._generate_binop(x, y, _add_vec)
-            elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(x, y, _add_mat)
+        def _add(builder, x, y):
+            assert helpers.is_floating_point(x)
+            assert helpers.is_floating_point(y)
+            return builder.fadd(x, y)
 
         return _add
 
+    def visit_Sub(self, node):
+        def _sub(builder, x, y):
+            assert helpers.is_floating_point(x)
+            assert helpers.is_floating_point(y)
+            return builder.fsub(x, y)
+
+        return _sub
+
     def visit_Mult(self, node):
-        def _mul_vec(ctx, builder, u, v):
-            assert u.type == v.type
-
-            dim = len(u.type.pointee)
-            output_vec = builder.alloca(u.type.pointee)
-
-            # Get the pointer to the first element of the array to convert from [? x double]* -> double*
-            vec_u = builder.gep(u, [ctx.int32_ty(0), ctx.int32_ty(0)])
-            vec_v = builder.gep(v, [ctx.int32_ty(0), ctx.int32_ty(0)])
-            vec_out = builder.gep(output_vec, [ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_vec_hadamard")
-            builder.call(builtin, [vec_u, vec_v, ctx.int32_ty(dim), vec_out])
-            return output_vec
-
-        def _mul_mat(ctx, builder, m1, m2):
-            assert m1.type == m2.type
-
-            x = len(m1.type.pointee)
-            y = len(m1.type.pointee.element)
-
-            output_mat = builder.alloca(m1.type.pointee)
-
-            m1_ptr = builder.gep(m1, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-            m2_ptr = builder.gep(m2, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-            output_ptr = builder.gep(output_mat, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_mat_hadamard")
-            builder.call(builtin, [m1_ptr, m2_ptr, ctx.int32_ty(x), ctx.int32_ty(y), output_ptr])
-            return output_mat
-
-        def _mul_mat_scalar(ctx, builder, m1, s):
-            x = len(m1.type.pointee)
-            y = len(m1.type.pointee.element)
-            output_mat = builder.alloca(m1.type.pointee)
-
-            m1_ptr = builder.gep(m1, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-            output_ptr = builder.gep(output_mat, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_mat_scalar_mult")
-            builder.call(builtin, [m1_ptr, s, ctx.int32_ty(x), ctx.int32_ty(y), output_ptr])
-            return output_mat
-
-        def _mul_vec_scalar(ctx, builder, v, s):
-            x = len(v.type.pointee)
-            output_vec = builder.alloca(v.type.pointee)
-
-            v_ptr = builder.gep(v, [ctx.int32_ty(0), ctx.int32_ty(0)])
-            output_ptr = builder.gep(output_vec, [ctx.int32_ty(0), ctx.int32_ty(0)])
-
-            builtin = ctx.import_llvm_function("__pnl_builtin_vec_scalar_mult")
-            builder.call(builtin, [v_ptr, s, ctx.int32_ty(x), output_ptr])
-            return output_vec
-
-        def _mul(x, y):
-            if helpers.is_floating_point(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fmul(x,y))
-            elif helpers.is_vector(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _mul_vec_scalar)
-            elif helpers.is_floating_point(x) and helpers.is_vector(y):
-                return self._generate_binop(y, x, _mul_vec_scalar)
-            elif helpers.is_2d_matrix(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _mul_mat_scalar)
-            elif helpers.is_floating_point(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(y, x, _mul_mat_scalar)
-            elif helpers.is_vector(x) and helpers.is_vector(y):
-                if x.type != y.type:
-                    # Special case: Cast y into scalar if it can be done
-                    if helpers.get_array_shape(y) == [1]:
-                        y = self.builder.gep(y, [self.ctx.int32_ty(0), self.ctx.int32_ty(0)])
-                        return self._generate_binop(x, y, _mul_vec_scalar)
-                return self._generate_binop(x, y, _mul_vec)
-            elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(x, y, _mul_mat)
+        def _mul(builder, x, y):
+            assert helpers.is_floating_point(x)
+            assert helpers.is_floating_point(y)
+            return builder.fmul(x, y)
 
         return _mul
 
     def visit_Div(self, node):
-        def _div_array(ctx, builder, u, v):
-            assert u.type == v.type
-            output_ptr = builder.alloca(u.type.pointee)
+        def _div(builder, x, y):
+            assert helpers.is_floating_point(x)
+            assert helpers.is_floating_point(y)
+            return builder.fdiv(x, y)
 
-            for (u_ptr, v_ptr, out_ptr) in helpers.recursive_iterate_arrays(ctx, builder, u, v, output_ptr):
-                u_val = builder.load(u_ptr)
-                v_val = builder.load(v_ptr)
-                builder.store(builder.fdiv(u_val, v_val), out_ptr)
-
-            return output_ptr
-
-        def _div_array_scalar(ctx, builder, array, s):
-            output_ptr = builder.alloca(array.type.pointee)
-            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fdiv(x, s), output_ptr)
-
-            return output_ptr
-
-        def _div_scalar_array(ctx, builder, s, array):
-            output_ptr = builder.alloca(array.type.pointee)
-            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fdiv(s, x), output_ptr)
-
-            return output_ptr
-
-        def _div(x, y):
-            if helpers.is_floating_point(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fdiv(x, y))
-            elif helpers.is_floating_point(x) and (helpers.is_2d_matrix(y) or helpers.is_vector(y)):
-                return self._generate_binop(x, y, _div_scalar_array)
-            elif helpers.is_vector(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _div_array_scalar)
-            elif helpers.is_2d_matrix(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _div_array_scalar)
-            elif helpers.is_vector(x) and helpers.is_vector(y):
-                if x.type != y.type:
-                    # Special case: Cast y into scalar if it can be done
-                    if helpers.get_array_shape(y) == [1]:
-                        y = self.builder.gep(y, [self.ctx.int32_ty(0), self.ctx.int32_ty(0)])
-                        return self._generate_binop(x, y, _div_array_scalar)
-                return self._generate_binop(x, y, _div_array)
-            elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(x, y, _div_array)
-            assert False, f"Unable to divide arguments {x}, {y}"
         return _div
 
-    def _generate_unop(self, x, callback):
-        if helpers.is_floating_point(x) and helpers.is_pointer(x):
-            x = self.builder.load(x)
+    def visit_Pow(self, node):
+        def _div(builder, x, y):
+            assert helpers.is_floating_point(x)
+            assert helpers.is_floating_point(y)
+            pow_f = ctx.get_builtin("pow", [x.type, y.type])
+            return builder.call(pow_f, [x, y])
 
-        return callback(self.ctx, self.builder, x)
+        return _div
 
     def visit_USub(self, node):
-        def _usub_vec_mat(ctx, builder, x):
-            output = builder.alloca(x.type.pointee)
-
-            helpers.call_elementwise_operation(ctx, builder, x, lambda ctx, builder, x: helpers.fneg(builder, x), output)
-            return output
-
-        def _usub(x):
-            if helpers.is_floating_point(x):
-                return self._generate_unop(x, lambda ctx, builder, x: helpers.fneg(builder, x))
-            elif helpers.is_vector(x) or helpers.is_2d_matrix(x):
-                return self._generate_unop(x, _usub_vec_mat)
+        def _usub(builder, x):
+            assert helpers.is_floating_point(x)
+            return helpers.fneg(builder, x)
 
         return _usub
 
+    def visit_UAdd(self, node):
+        def _uadd(builder, x):
+            assert helpers.is_floating_point(x)
+            return x
+
+        return _uadd
+
+    def visit_Not(self, node):
+        def _not(builder, x):
+            assert helpers.is_scalar(x)
+            # The result of 'not' is always bool even if the input is not
+            x_b = helpers.convert_type(builder, x, self.ctx.bool_ty)
+            return builder.not_(x_b)
+
+        return _not
+
     def visit_Name(self, node):
-        return self.register[node.id]
+        return self.register.get(node.id, None)
 
     def visit_Attribute(self, node):
         val = self.visit(node.value)
@@ -377,42 +174,26 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
             shape = helpers.get_array_shape(val)
             return ir.ArrayType(self.ctx.float_ty, len(shape))(shape)
         elif node.attr == "flatten":
-            def flatten():
-                shape = helpers.get_array_shape(val)
-                flattened_size = reduce(lambda x, y: x * y, shape)
-                flattened_ty = ir.ArrayType(self.ctx.float_ty, flattened_size)
-                flattened_array = self.builder.alloca(flattened_ty)
-                index_var = self.builder.alloca(self.ctx.int32_ty, name="flattened_index_var_loc")
-                self.builder.store(self.ctx.int32_ty(0), index_var)
-                for (array_ptr,) in helpers.recursive_iterate_arrays(self.ctx, self.builder, val):
-                    index = self.builder.load(index_var, name="flattened_index_var")
-                    flattened_array_ptr = self.builder.gep(flattened_array, [self.ctx.int32_ty(0), index])
-                    array_val = self.builder.load(array_ptr)
-                    self.builder.store(array_val, flattened_array_ptr)
-                    index = self.builder.add(index, self.ctx.int32_ty(1), name="flattened_index_var_inc")
-                    self.builder.store(index, index_var)
-                return flattened_array
+            val = self.get_rval(val)
+            def flatten(builder):
+                res = []
+                def collect(builder, x):
+                    res.append(x)
+                    return x
+                self._do_unary_op(builder, val, collect)
+
+                assert len(res) > 0
+                flat = ir.ArrayType(res[0].type, len(res))(ir.Undefined)
+                for i, v in enumerate(res):
+                    flat = self.builder.insert_value(flat, v, i)
+                return flat
             return flatten
         elif node.attr == "astype":
-            def astype(ty):
-                def _convert(ctx, builder, x):
-                    if helpers.is_pointer(x):
-                        x = builder.load(x)
-                    if helpers.is_integer(x) and ty is ctx.float_ty:
-                        if helpers.is_boolean(x):
-                            return builder.uitofp(x, ty)
-                        return builder.sitofp(x, ty)
-                    elif helpers.is_floating_point(x) and ty is self.register["int"]:
-                        return builder.fptosi(x, ty)
-                    elif (helpers.is_floating_point(x) and ty is ctx.float_ty):
-                        return x
-                if helpers.is_scalar(val):
-                    return _convert(self.ctx, self.builder, val)
-                else:
-                    output_ptr = self.builder.alloca(helpers.array_from_shape(helpers.get_array_shape(val), ty))
-                    helpers.call_elementwise_operation(self.ctx, self.builder, val, _convert, output_ptr)
-                    return output_ptr
-            # we only support float types
+            val = self.get_rval(val)
+            def astype(builder, ty):
+                def _convert(builder, x):
+                    return helpers.convert_type(builder, x, ty)
+                return self._do_unary_op(builder, val, _convert)
             return astype
 
         return val[node.attr]
@@ -421,19 +202,15 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
         return self.ctx.float_ty(node.n)
 
     def visit_Assign(self, node):
-        value = self.visit(node.value)
-        def _assign_target(target, value):
-            if isinstance(target, ast.Name):
-                id = target.id
-                self.register[id] = value
-            else:
-                to_store = value
+        value = self.get_rval(self.visit(node.value))
 
-                target = self.visit(target)
-                self.builder.store(to_store, target)
-
-        for target in node.targets:
-            _assign_target(target, value)
+        for t in node.targets:
+            target = self.visit(t)
+            if target is None: # Allocate space for new variable
+                target = self.var_builder.alloca(value.type)
+                self.register[t.id] = target
+            assert self.is_lval(target)
+            self.builder.store(value, target)
 
     def visit_NameConstant(self, node):
         val = self.name_constants[node.value]
@@ -441,113 +218,119 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
         return val
 
     def visit_Tuple(self, node):
-        elements = [self.visit(element) for element in node.elts]
-
-        elements = [self.builder.load(element) if helpers.is_pointer(element) else element for element in elements]
-
-        element_types = [element.type for element in elements]
+        elements = (self.visit(element) for element in node.elts)
+        element_values = [self.builder.load(element) if helpers.is_pointer(element) else element for element in elements]
+        element_types = [element.type for element in element_values]
         if len(element_types) > 0 and all(x == element_types[0] for x in element_types):
-            ret_list = self.builder.alloca(ir.ArrayType(element_types[0], len(element_types)))
+            result = ir.ArrayType(element_types[0], len(element_types))(ir.Undefined)
         else:
-            ret_list = self.builder.alloca(ir.LiteralStructType(element_types))
+            result = ir.LiteralStructType(element_types)(ir.Undefined)
 
-        for idx, element in enumerate(elements):
-            self.builder.store(element, self.builder.gep(ret_list, [self.ctx.int32_ty(0), self.ctx.int32_ty(idx)]))
+        for i, val in enumerate(element_values):
+            result = self.builder.insert_value(result, val, i)
 
-        return ret_list
+        return result
 
-    def visit_BinOp(self, node):
-        operator = self.visit(node.op)
-        return operator(self.visit(node.left), self.visit(node.right))
+    def _do_unary_op(self, builder, x, scalar_op):
+        assert not helpers.is_pointer(x)
 
-    def visit_BoolOp(self, node):
-        operator = self.visit(node.op)
-        values = [self.visit(value) for value in node.values]
-        ret_val = values[0]
-        for value in values[1:]:
-            ret_val = operator(ret_val, value)
-        return ret_val
+        # scalar is the base case
+        if helpers.is_scalar(x):
+            return scalar_op(self.builder, x)
+        operands = (builder.extract_value(x, i) for i in range(len(x.type)))
+        results = [self._do_unary_op(builder, opx, scalar_op) for opx in operands]
+
+        result = ir.ArrayType(results[0].type, len(results))(ir.Undefined)
+        for i, res in enumerate(results):
+            result = builder.insert_value(result, res, i)
+
+        return result
 
     def visit_UnaryOp(self, node):
         operator = self.visit(node.op)
-        return operator(self.visit(node.operand))
+        operand = self.get_rval(self.visit(node.operand))
+        return self._do_unary_op(self.builder, operand, operator)
 
-    def visit_List(self, node):
-        elements = [self.visit(element) for element in node.elts]
-        element_ty = elements[0].type
-        assert all(element.type == element_ty for element in elements), f"Unable to convert {node} into a list! (Elements differ in type!)"
+    def _do_bin_op(self, builder, x, y, scalar_op):
+        assert not helpers.is_pointer(x)
+        assert not helpers.is_pointer(y)
 
-        # dereference pointers
-        if helpers.is_pointer(element_ty):
-            elements = [self.builder.load(element) for element in elements]
-            element_ty = elements[0].type
+        # 2 scalars is the base case
+        if helpers.is_scalar(x) and helpers.is_scalar(y):
+            return scalar_op(self.builder, x, y)
 
-        ret_list = self.builder.alloca(ir.ArrayType(element_ty, len(elements)))
+        x_len = len(x.type) if hasattr(x.type, '__len__') else 0
+        y_len = len(y.type) if hasattr(y.type, '__len__') else 0
+        iters = max(x_len, y_len)
+        assert x_len == 0 or x_len == iters
+        assert y_len == 0 or y_len == iters
+        assert iters > 0, "At least one of the operands should be vector: {} op {}".format(x.type, y.type)
 
-        for idx, element in enumerate(elements):
-            self.builder.store(element, self.builder.gep(ret_list, [self.ctx.int32_ty(0), self.ctx.int32_ty(idx)]))
-        return ret_list
+        x_operands = (builder.extract_value(x, i) if x_len > 0 else x for i in range(iters))
+        y_operands = (builder.extract_value(y, i) if y_len > 0 else y for i in range(iters))
+        results = [self._do_bin_op(builder, opx, opy, scalar_op) for opx, opy in zip(x_operands, y_operands)]
+
+        assert len(results) > 0
+        res = ir.ArrayType(results[0].type, len(results))(ir.Undefined)
+        for i in range(iters):
+            res = builder.insert_value(res, results[i], i)
+
+        return res
+
+    def visit_BinOp(self, node):
+        operator = self.visit(node.op)
+        lhs = self.get_rval(self.visit(node.left))
+        rhs = self.get_rval(self.visit(node.right))
+        return self._do_bin_op(self.builder, lhs, rhs, operator)
+
+    def visit_BoolOp(self, node):
+        operator = self.visit(node.op)
+        values = (self.get_rval(self.visit(value)) for value in node.values)
+        ret_val = next(values)
+        for value in values:
+            assert ret_val.type == value.type, "Don't know how to mix types in boolean expressions!"
+            ret_val = operator(self.builder, ret_val, value)
+        return ret_val
 
     def visit_And(self, node):
-        def _and(x, y):
-            assert helpers.is_boolean(x), f"{x} is not a Boolean!"
-            assert helpers.is_boolean(y), f"{y} is not a Boolean!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.and_(x, y))
+        def _and(builder, x, y):
+            # Python's 'and' takes the boolean value of the first operand and returns
+            # RHS if bool(LHS) else LHS
+            cond = helpers.convert_type(builder, x, self.ctx.bool_ty)
+            return builder.select(cond, y, x)
 
         return _and
 
     def visit_Or(self, node):
-        def _or(x, y):
-            assert helpers.is_boolean(x), f"{x} is not a Boolean!"
-            assert helpers.is_boolean(y), f"{y} is not a Boolean!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.or_(x, y))
+        def _or(builder, x, y):
+            # Python's 'or' takes the boolean value of the first operand and returns
+            # LHS if bool(LHS) else RHS
+            cond = helpers.convert_type(builder, x, self.ctx.bool_ty)
+            return builder.select(cond, x, y)
 
         return _or
 
-    def _generate_fcmp_handler(self, ctx, builder, cmp):
-        def _cmp_array(ctx, builder, u, v):
-            assert u.type == v.type
-            shape = helpers.get_array_shape(u)
-            output_ptr = builder.alloca(helpers.array_from_shape(shape, ctx.bool_ty))
+    def visit_List(self, node):
+        element_values = [self.get_rval(self.visit(element)) for element in node.elts]
+        element_types = [element.type for element in element_values]
+        assert all(e_type == element_types[0] for e_type in element_types), f"Unable to convert {node} into a list! (Elements differ in type!)"
+        result = ir.ArrayType(element_types[0], len(element_types))(ir.Undefined)
 
-            for (u_ptr, v_ptr, out_ptr) in helpers.recursive_iterate_arrays(ctx, builder, u, v, output_ptr):
-                u_val = builder.load(u_ptr)
-                v_val = builder.load(v_ptr)
-                builder.store(builder.fcmp_ordered(cmp, u_val, v_val), out_ptr)
+        for i, val in enumerate(element_values):
+            result = self.builder.insert_value(result, val, i)
 
-            return output_ptr
+        return result
 
-        def _cmp_array_scalar(ctx, builder, array, s):
-            shape = helpers.get_array_shape(array)
-            output_ptr = builder.alloca(helpers.array_from_shape(shape, ctx.bool_ty))
-            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fcmp_ordered(cmp, x, s), output_ptr)
+    def _generate_fcmp_handler(self, ctx, builder, cmp_op):
+        def _fcmp(builder, x, y):
+            # If at least one operand if float, we cast the other one to float as well
+            assert helpers.is_floating_point(x) or helpers.is_floating_point(y)
+            float_ty = x.type if helpers.is_floating_point(x) else y.type
+            x = helpers.convert_type(builder, x, float_ty)
+            y = helpers.convert_type(builder, y, float_ty)
+            return builder.fcmp_ordered(cmp_op, x, y)
 
-            return output_ptr
-
-        def _cmp_scalar_array(ctx, builder, s, array):
-            shape = helpers.get_array_shape(array)
-            output_ptr = builder.alloca(helpers.array_from_shape(shape, ctx.bool_ty))
-            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fcmp_ordered(cmp, s, x), output_ptr)
-
-            return output_ptr
-
-        def _cmp(x, y):
-            if helpers.is_floating_point(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered(cmp, x, y))
-            elif helpers.is_vector(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _cmp_array_scalar)
-            elif helpers.is_floating_point(x) and helpers.is_vector(y):
-                return self._generate_binop(x, y, _cmp_scalar_array)
-            elif helpers.is_2d_matrix(x) and helpers.is_floating_point(y):
-                return self._generate_binop(x, y, _cmp_array_scalar)
-            elif helpers.is_floating_point(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(x, y, _cmp_scalar_array)
-            elif helpers.is_vector(x) and helpers.is_vector(y):
-                return self._generate_binop(x, y, _cmp_array)
-            elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
-                return self._generate_binop(x, y, _cmp_array)
-
-        return _cmp
+        return _fcmp
 
     def visit_Eq(self, node):
         return self._generate_fcmp_handler(self.ctx, self.builder, "==")
@@ -568,15 +351,19 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
         return self._generate_fcmp_handler(self.ctx, self.builder, ">=")
 
     def visit_Compare(self, node):
-        comp_val = self.visit(node.left)
-        comparators = [self.visit(comparator) for comparator in node.comparators]
-        ops = [self.visit(op) for op in node.ops]
-        for comparator, op in zip(comparators, ops):
-            comp_val = op(comp_val, comparator)
-        return comp_val
+        result = self.get_rval(self.visit(node.left))
+
+        comparators = (self.visit(comparator) for comparator in node.comparators)
+        values = (self.builder.load(val) if helpers.is_pointer(val) else val for val in comparators)
+        ops = (self.visit(op) for op in node.ops)
+        for val, op in zip(values, ops):
+            result = self._do_bin_op(self.builder, result, val, op)
+        return result
 
     def visit_If(self, node):
-        predicate = self.visit(node.test)
+        cond_val = self.get_rval(self.visit(node.test))
+
+        predicate = helpers.convert_type(self.builder, cond_val, self.ctx.bool_ty)
         with self.builder.if_else(predicate) as (then, otherwise):
             with then:
                 for child in node.body:
@@ -595,27 +382,116 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
 
         # get position in arg_out if types differ
         if (helpers.is_scalar(ret_val) or helpers.is_vector(ret_val)) and helpers.is_2d_matrix(arg_out):
+            assert len(arg_out.type.pointee) == 1
             arg_out = self.builder.gep(arg_out, [self.ctx.int32_ty(0), self.ctx.int32_ty(0)])
 
         if helpers.is_scalar(ret_val) and helpers.is_vector(arg_out):
+            assert len(arg_out.type.pointee) == 1
             arg_out = self.builder.gep(arg_out, [self.ctx.int32_ty(0), self.ctx.int32_ty(0)])
 
         self.builder.store(ret_val, arg_out)
-
-    def visit_Call(self, node):
-        call_func = self.visit(node.func)
-        assert callable(call_func), f"Uncallable function {node.func}!"
-        node_args = [self.visit(arg) for arg in node.args]
-
-        return call_func(*node_args)
+        self.builder.ret_void()
 
     def visit_Subscript(self, node):
         node_val = self.visit(node.value)
-        node_slice_val = self.visit(node.slice)
+        index = self.visit(node.slice)
+        node_slice_val = helpers.convert_type(self.builder, index, self.ctx.int32_ty)
+        if not self.is_lval(node_val):
+            temp_node_val = self.builder.alloca(node_val.type)
+            self.builder.store(node_val, temp_node_val)
+            node_val = temp_node_val
+
         return self.builder.gep(node_val, [self.ctx.int32_ty(0), node_slice_val])
 
     def visit_Index(self, node):
-        return self.builder.fptoui(self.visit(node.value), self.ctx.int32_ty)
+        """
+        Returns the wrapped value.
+
+        Deprecated in python 3.9+.
+        """
+        return self.visit(node.value)
+
+    def visit_Call(self, node):
+        node_args = [self.visit(arg) for arg in node.args]
+
+        call_func = self.visit(node.func)
+        assert callable(call_func), f"Uncallable function {node.func}!"
+
+        return call_func(self.builder, *node_args)
+
+    # Python builtins
+
+    def call_builtin_horizontal_sum(self, builder, x):
+        # HACK: Get scalar version of add
+        add_func = self.visit_Add(None)
+
+        if helpers.is_pointer(x):
+            x = builder.load(x)
+        vals = (builder.extract_value(x, i) for i in range(len(x.type)))
+
+        total_sum = next(vals)
+        for val in vals:
+            total_sum = self._do_bin_op(builder, total_sum, val, add_func)
+        return total_sum
+
+    def call_builtin_len(self, builder, x):
+        x_ty = x.type
+        if helpers.is_pointer(x):
+            x_ty = x_ty.pointee
+        return self.ctx.float_ty(len(x_ty))
+
+    def call_builtin_max(self, builder, *args):
+        # Python max takes 1 iterable (array in our case),
+        # or a set of multiple arguments.
+        # Unlike numpy is searches only in the first dimension
+        # see: https://docs.python.org/3/library/functions.html#max
+
+        assert len(args) > 0, "Empty argument list for 'max'!"
+        if len(args) == 1:
+            assert helpers.is_vector(args[0]), "Only 1D vectors supported!"
+            arg = builder.load(args[0]) if helpers.is_pointer(args[0]) else args[0]
+            values = (builder.extract_value(arg, i) for i in range(len(arg.type)))
+        else:
+            values = (builder.load(arg) if helpers.is_pointer(arg) else arg for arg in args)
+
+        res = next(values)
+        for val in values:
+            greater = builder.fcmp_ordered('>', val, res)
+            res = builder.select(greater, val, res)
+        return res
+
+    #  Numpy builtins
+
+    def call_builtin_np_tanh(self, builder, x):
+        x = self.get_rval(x)
+        return self._do_unary_op(builder, x, lambda builder, x: helpers.tanh(self.ctx, builder, x))
+
+    def call_builtin_np_exp(self, builder, x):
+        x = self.get_rval(x)
+        return self._do_unary_op(builder, x, lambda builder, x: helpers.exp(self.ctx, builder, x))
+
+    def call_builtin_np_max(self, builder, x):
+        # numpy max searches for the largest scalar and propagates NaNs be default.
+        # Only the default behaviour is supported atm
+        # see: https://numpy.org/doc/stable/reference/generated/numpy.amax.html#numpy.amax
+
+        x = self.get_rval(x)
+        if helpers.is_scalar(x):
+            return x
+        res = self.ctx.float_ty("-Inf")
+        def find_max(builder, x):
+            nonlocal res
+            # to propagate NaNs we use unordered >,
+            # but only update if the current result is not NaN
+            not_nan = builder.fcmp_ordered('ord', res, res)
+            greater = builder.fcmp_unordered('>', x, res)
+            cond = builder.and_(not_nan, greater)
+            res = builder.select(cond, x, res)
+            return res
+        self._do_unary_op(builder, x, find_max)
+        return res
+
+
 
 def gen_node_wrapper(ctx, composition, node, *, tags:frozenset):
     assert "node_wrapper" in tags
