@@ -13,11 +13,12 @@ Interface to PNL_LLVM_DEBUG environment variable.
 
 The currently recognized values are:
 Features:
- * "cuda" -- enable execution on CUDA devices if available
+ * "cuda-check" -- print the result of initializing pycuda
 
 Increased debug output:
  * "compile" -- prints information messages when modules are compiled
  * "stat" -- prints code generation and compilation statistics
+ * "time_stat" -- print compilation and code generation times
  * "cuda_data" -- print data upload/download statistic (to GPU VRAM)
  * "comp_node_debug" -- print intermediate results after execution composition node wrapper.
  * "print_values" -- Enabled printfs in llvm code (from ctx printf helper)
