@@ -8,13 +8,15 @@
 
 # ********************************************* PNL LLVM helpers **************************************************************
 
-from llvmlite import ir
 from contextlib import contextmanager
 from ctypes import util
 
-from ..scheduling.condition import All, AllHaveRun, Always, Any, AtPass, AtTrial, BeforeNCalls, AtNCalls, AfterNCalls, EveryNCalls, Never, Not, WhenFinished, WhenFinishedAny, WhenFinishedAll
-from ..scheduling.time import TimeScale
+from llvmlite import ir
+
 from .debug import debug_env
+from psyneulink.core.scheduling.condition import All, AllHaveRun, Always, Any, AtPass, AtTrial, BeforeNCalls, AtNCalls, AfterNCalls, \
+    EveryNCalls, Never, Not, WhenFinished, WhenFinishedAny, WhenFinishedAll
+from psyneulink.core.scheduling.time import TimeScale
 
 
 @contextmanager
@@ -130,51 +132,34 @@ def fneg(builder, val, name=""):
     return builder.fsub(val.type(-0.0), val, name)
 
 
-def tanh(ctx, builder, x):
-    # (e**2x - 1)/(e**2x + 1)
-    _2x = builder.fmul(x.type(2), x)
-    e2x = exp(ctx, builder, _2x)
-    num = builder.fsub(e2x, e2x.type(1))
-    den = builder.fadd(e2x, e2x.type(1))
-    return builder.fdiv(num, den)
-
 def exp(ctx, builder, x):
     exp_f = ctx.get_builtin("exp", [x.type])
     return builder.call(exp_f, [x])
 
+
+def tanh(ctx, builder, x):
+    tanh_f = ctx.get_builtin("tanh", [x.type])
+    return builder.call(tanh_f, [x])
+
+
 def coth(ctx, builder, x):
-    # (e**2x + 1)/(e**2x - 1)
-    _2x = builder.fmul(x.type(2), x)
-    e2x = exp(ctx, builder, _2x)
-    num = builder.fadd(e2x, e2x.type(1))
-    den = builder.fsub(e2x, e2x.type(1))
-    return builder.fdiv(num, den)
+    coth_f = ctx.get_builtin("coth", [x.type])
+    return builder.call(coth_f, [x])
 
 
 def csch(ctx, builder, x):
-    # (2e**x)/(e**2x - 1)
-    ex = exp(ctx, builder, x)
-    num = builder.fmul(ex.type(2), ex)
-    _2x = builder.fmul(x.type(2), x)
-    e2x = exp(ctx, builder, _2x)
-    den = builder.fsub(e2x, e2x.type(1))
-    return builder.fdiv(num, den)
-
-def is_close(builder, val1, val2, rtol=1e-05, atol=1e-08):
-    diff = builder.fsub(val1, val2, "is_close_diff")
-    diff_neg = fneg(builder, diff, "is_close_fneg_diff")
-    ltz = builder.fcmp_ordered("<", diff, diff.type(0.0), "is_close_ltz")
-    abs_diff = builder.select(ltz, diff_neg, diff, "is_close_abs")
-
-    rev2 = fneg(builder, val2, "is_close_fneg2")
-    ltz2 = builder.fcmp_ordered("<", val2, val2.type(0.0), "is_close_ltz2")
-    abs2 = builder.select(ltz2, rev2, val2, "is_close_abs2")
-    rtol = builder.fmul(abs2.type(rtol), abs2, "is_close_rtol")
-    atol = builder.fadd(rtol, rtol.type(atol), "is_close_atol")
-    return builder.fcmp_ordered("<=", abs_diff, atol, "is_close_cmp")
+    csch_f = ctx.get_builtin("csch", [x.type])
+    return builder.call(csch_f, [x])
 
 
-def all_close(builder, arr1, arr2, rtol=1e-05, atol=1e-08):
+def is_close(ctx, builder, val1, val2, rtol=1e-05, atol=1e-08):
+    is_close_f = ctx.get_builtin("is_close")
+    rtol_val = val1.type(rtol)
+    atol_val = val1.type(atol)
+    return builder.call(is_close_f, [val1, val2, rtol_val, atol_val])
+
+
+def all_close(ctx, builder, arr1, arr2, rtol=1e-05, atol=1e-08):
     assert arr1.type == arr2.type
     all_ptr = builder.alloca(ir.IntType(1))
     builder.store(all_ptr.type.pointee(1), all_ptr)
@@ -183,7 +168,7 @@ def all_close(builder, arr1, arr2, rtol=1e-05, atol=1e-08):
         val2_ptr = b1.gep(arr1, [idx.type(0), idx])
         val1 = b1.load(val1_ptr)
         val2 = b1.load(val2_ptr)
-        res_close = is_close(b1, val1, val2, rtol, atol)
+        res_close = is_close(ctx, b1, val1, val2, rtol, atol)
 
         all_val = b1.load(all_ptr)
         all_val = b1.and_(all_val, res_close)
@@ -191,8 +176,74 @@ def all_close(builder, arr1, arr2, rtol=1e-05, atol=1e-08):
 
     return builder.load(all_ptr)
 
+
+def create_allocation(builder, allocation, search_space, idx):
+    # Construct allocation corresponding to this index
+    for i in reversed(range(len(search_space.type.pointee))):
+        slot_ptr = builder.gep(allocation, [idx.type(0), idx.type(i)])
+
+        dim_ptr = builder.gep(search_space, [idx.type(0), idx.type(i)])
+        # Iterators store {start, step, num}
+        if isinstance(dim_ptr.type.pointee,  ir.LiteralStructType):
+            iter_val = builder.load(dim_ptr)
+            dim_start = builder.extract_value(iter_val, 0)
+            dim_step = builder.extract_value(iter_val, 1)
+            dim_size = builder.extract_value(iter_val, 2)
+            dim_idx = builder.urem(idx, dim_size)
+            val = builder.uitofp(dim_idx, dim_step.type)
+            val = builder.fmul(val, dim_step)
+            val = builder.fadd(val, dim_start)
+        elif isinstance(dim_ptr.type.pointee,  ir.ArrayType):
+            # Otherwise it's just an array
+            dim_size = idx.type(len(dim_ptr.type.pointee))
+            dim_idx = builder.urem(idx, dim_size)
+            val_ptr = builder.gep(dim_ptr, [idx.type(0), dim_idx])
+            val = builder.load(val_ptr)
+        else:
+            assert False, "Unknown dimension type: {}".format(dim_ptr.type)
+
+        idx = builder.udiv(idx, dim_size)
+
+        builder.store(val, slot_ptr)
+
+
+def convert_type(builder, val, t):
+    assert isinstance(t, ir.Type)
+    if val.type == t:
+        return val
+
+    if is_floating_point(val) and is_boolean(t):
+        # convert any scalar to bool by comparing to 0
+        # Python converts both 0.0 and -0.0 to False
+        return builder.fcmp_unordered("!=", val, val.type(0.0))
+
+    if is_boolean(val) and is_floating_point(t):
+        # float(True) == 1.0, float(False) == 0.0
+        return builder.select(val, t(1.0), t(0.0))
+
+    if is_integer(val) and is_integer(t):
+        if val.type.width > t.width:
+            return builder.trunc(val, t)
+        elif val.type.width < t.width:
+            # Python integers are signed
+            return builder.sext(val, t)
+        else:
+            assert False, "Unknown integer conversion: {} -> {}".format(val.type, t)
+
+    if is_integer(val) and is_floating_point(t):
+        # Python integers are signed
+        return builder.sitofp(val, t)
+
+    if is_floating_point(val) and is_integer(t):
+        # Python integers are signed
+        return builder.fptosi(val, t)
+
+    assert False, "Unknown type conversion: {} -> {}".format(val.type, t)
+
+
 def is_pointer(x):
     type_t = getattr(x, "type", x)
+    assert isinstance(type_t, ir.Type)
     return isinstance(type_t, ir.PointerType)
 
 def is_floating_point(x):
