@@ -365,24 +365,24 @@ from collections.abc import Iterable
 import numpy as np
 import typecheck as tc
 
-from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import \
+from psyneulink.core.components.functions.function import DEFAULT_SEED, _random_state_getter, _seed_setter
+from psyneulink.core.components.functions.stateful.integratorfunctions import \
     DriftDiffusionIntegrator, IntegratorFunction
-from psyneulink.core.components.functions.distributionfunctions import STARTING_POINT, \
+from psyneulink.core.components.functions.nonstateful.distributionfunctions import STARTING_POINT, \
     DriftDiffusionAnalytical
-from psyneulink.core.components.functions.combinationfunctions import Reduce
+from psyneulink.core.components.functions.nonstateful.combinationfunctions import Reduce
 from psyneulink.core.components.mechanisms.modulatory.control.controlmechanism import _is_control_spec
-from psyneulink.core.components.mechanisms.mechanism import Mechanism_Base
 from psyneulink.core.components.mechanisms.processing.processingmechanism import ProcessingMechanism
 from psyneulink.core.components.ports.modulatorysignals.controlsignal import ControlSignal
 from psyneulink.core.components.ports.outputport import SEQUENTIAL, StandardOutputPorts
 from psyneulink.core.globals.context import ContextFlags, handle_external_context
 from psyneulink.core.globals.keywords import \
-    ALLOCATION_SAMPLES, FUNCTION, FUNCTION_PARAMS, INPUT_PORT_VARIABLES, NAME, OUTPUT_PORTS, OWNER_VALUE, \
+    ALLOCATION_SAMPLES, FUNCTION, FUNCTION_PARAMS, INPUT_PORT_VARIABLES, NAME, OWNER_VALUE, \
     THRESHOLD, VARIABLE, PREFERENCE_SET_NAME
 from psyneulink.core.globals.parameters import Parameter
 from psyneulink.core.globals.preferences.basepreferenceset import is_pref_set, REPORT_OUTPUT_PREF
 from psyneulink.core.globals.preferences.preferenceset import PreferenceEntry, PreferenceLevel
-from psyneulink.core.globals.utilities import convert_to_np_array, is_numeric, is_same_function_spec, object_has_single_value, get_global_seed
+from psyneulink.core.globals.utilities import convert_all_elements_to_np_array, is_numeric, is_same_function_spec, object_has_single_value
 
 from psyneulink.core import llvm as pnlvm
 
@@ -717,7 +717,8 @@ class DDM(ProcessingMechanism):
         )
         input_format = Parameter(SCALAR, stateful=False, loggable=False)
         initializer = np.array([[0]])
-        random_state = Parameter(None, stateful=True, loggable=False)
+        random_state = Parameter(None, loggable=False, getter=_random_state_getter, dependencies='seed')
+        seed = Parameter(DEFAULT_SEED, modulable=True, setter=_seed_setter)
 
         output_ports = Parameter(
             [DECISION_VARIABLE, RESPONSE_TIME],
@@ -758,9 +759,6 @@ class DDM(ProcessingMechanism):
         #    in order to use SEQUENTIAL indices
         self.standard_output_ports = StandardOutputPorts(self, self.standard_output_ports, indices=SEQUENTIAL)
 
-        if seed is None:
-            seed = get_global_seed()
-
         if input_format is not None and input_ports is not None:
             raise DDMError(
                 'Only one of input_format and input_ports should be specified.'
@@ -819,9 +817,6 @@ class DDM(ProcessingMechanism):
         if isinstance(output_ports, (str, tuple)):
             output_ports = list(output_ports)
 
-        # Instantiate RandomState
-        random_state = np.random.RandomState([seed])
-
         # IMPLEMENTATION NOTE: this manner of setting default_variable works but is idiosyncratic
         # compared to other mechanisms: see TransferMechanism.py __init__ function for a more normal example.
         if default_variable is None and size is None:
@@ -840,7 +835,7 @@ class DDM(ProcessingMechanism):
         self.parameters.execute_until_finished.default_value = False
 
         super(DDM, self).__init__(default_variable=default_variable,
-                                  random_state=random_state,
+                                  seed=seed,
                                   input_ports=input_ports,
                                   output_ports=output_ports,
                                   function=function,
@@ -1047,7 +1042,7 @@ class DDM(ProcessingMechanism):
             if self.initialization_status != ContextFlags.INITIALIZING:
                 logger.info('{0} {1} is at {2}'.format(type(self).__name__, self.name, result))
 
-            return convert_to_np_array([result[0], [result[1]]])
+            return convert_all_elements_to_np_array([result[0], result[1]])
 
         # EXECUTE ANALYTIC SOLUTION (TRIAL TIME SCALE) -----------------------------------------------------------
         else:
@@ -1101,8 +1096,7 @@ class DDM(ProcessingMechanism):
             builder.store(builder.load(builder.gep(mf_out, [ctx.int32_ty(0),
                                                             ctx.int32_ty(1)])),
                           builder.gep(mech_out, [ctx.int32_ty(0),
-                                                 ctx.int32_ty(1),
-                                                 ctx.int32_ty(0)]))
+                                                 ctx.int32_ty(1)]))
         elif isinstance(self.function, DriftDiffusionAnalytical):
             for res_idx, idx in enumerate((self.RESPONSE_TIME_INDEX,
                                            self.PROBABILITY_LOWER_THRESHOLD_INDEX,
@@ -1145,7 +1139,7 @@ class DDM(ProcessingMechanism):
                 ctx.int32_ty(self.DECISION_VARIABLE_INDEX),
                 ctx.int32_ty(0)])
             thr_cmp = builder.fcmp_ordered("<", random_val, prob_lower_thr)
-            neg_threshold = builder.fsub(threshold.type(-0.0), threshold)
+            neg_threshold = pnlvm.helpers.fneg(builder, threshold)
             res = builder.select(thr_cmp, neg_threshold, threshold)
 
             builder.store(res, dst)
@@ -1156,12 +1150,12 @@ class DDM(ProcessingMechanism):
 
     @handle_external_context(fallback_most_recent=True)
     def reset(self, *args, force=False, context=None, **kwargs):
-        from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import IntegratorFunction
+        from psyneulink.core.components.functions.stateful.integratorfunctions import IntegratorFunction
 
         # (1) reset function, (2) update mechanism value, (3) update output ports
         if isinstance(self.function, IntegratorFunction):
             new_values = self.function.reset(*args, **kwargs, context=context)
-            self.parameters.value._set(convert_to_np_array(new_values), context)
+            self.parameters.value._set(convert_all_elements_to_np_array(new_values), context)
             self._update_output_ports(context=context)
 
     @handle_external_context()
@@ -1207,7 +1201,7 @@ class DDM(ProcessingMechanism):
             return ctx.bool_ty(1)
 
         # Extract scalar value from ptr
-        prev_val_ptr = builder.gep(prev_val_ptr, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
+        prev_val_ptr = builder.gep(prev_val_ptr, [ctx.int32_ty(0), ctx.int32_ty(0)])
         prev_val = builder.load(prev_val_ptr)
 
         # Take abs of previous val
