@@ -1,13 +1,6 @@
 from .regressioncfa import *
 from .compositionrunner import *
+from .autodiffcomposition import *
 __all__ = list(regressioncfa.__all__)
 __all__.extend(compositionrunner.__all__)
-
-try:
-    import torch
-
-    from .autodiffcomposition import *
-
-    __all__.extend(autodiffcomposition.__all__)
-except ImportError:
-    pass
+__all__.extend(autodiffcomposition.__all__)
