@@ -127,9 +127,10 @@ Class Reference
 ---------------
 
 """
+import logging
+
 import numpy as np
 
-import logging
 try:
     import torch
     from torch import nn
@@ -140,15 +141,14 @@ except ImportError:
 else:
     from psyneulink.library.compositions.pytorchmodelcreator import PytorchModelCreator
 
-
-from psyneulink.core.components.functions.transferfunctions import Linear, Logistic, ReLU
-from psyneulink.core.components.mechanisms.processing.compositioninterfacemechanism import CompositionInterfaceMechanism
 from psyneulink.library.components.mechanisms.processing.objective.comparatormechanism import ComparatorMechanism
-from psyneulink.core.components.projections.pathway.mappingprojection import MappingProjection
 from psyneulink.core.compositions.composition import Composition, NodeRole
 from psyneulink.core.compositions.composition import CompositionError
+from psyneulink.core.compositions.report \
+    import ReportOutput, ReportParams, ReportProgress, ReportSimulations, ReportDevices, \
+    LEARN_REPORT, EXECUTE_REPORT, PROGRESS_REPORT
 from psyneulink.core.globals.context import Context, ContextFlags, handle_external_context
-from psyneulink.core.globals.keywords import SOFT_CLAMP, TRAINING_SET
+from psyneulink.core.globals.keywords import SOFT_CLAMP
 from psyneulink.core.scheduling.scheduler import Scheduler
 from psyneulink.core.globals.parameters import Parameter
 from psyneulink.core.scheduling.time import TimeScale
@@ -231,6 +231,7 @@ class AutodiffComposition(Composition):
                  disable_cuda=True,
                  cuda_index=None,
                  force_no_retain_graph=False,
+                 pathways=None,
                  name="autodiff_composition"):
 
         if not torch_available:
@@ -241,7 +242,8 @@ class AutodiffComposition(Composition):
                                                   learning_rate = learning_rate,
                                                   optimizer_type = optimizer_type,
                                                   weight_decay = weight_decay,
-                                                  loss_spec = loss_spec)
+                                                  loss_spec = loss_spec,
+                                                  pathways=pathways)
 
         self.optimizer_type = optimizer_type
         self.loss_spec = loss_spec
@@ -477,8 +479,15 @@ class AutodiffComposition(Composition):
                 clamp_input=SOFT_CLAMP,
                 targets=None,
                 runtime_params=None,
-                bin_execute=False,
+                execution_mode:pnlvm.ExecutionMode = pnlvm.ExecutionMode.Python,
                 skip_initialization=False,
+                report_output:ReportOutput=ReportOutput.OFF,
+                report_params:ReportOutput=ReportParams.OFF,
+                report_progress:ReportProgress=ReportProgress.OFF,
+                report_simulations:ReportSimulations=ReportSimulations.OFF,
+                report_to_devices:ReportDevices=None,
+                report=None,
+                report_num=None,
                 ):
         self._assign_execution_ids(context)
         context.composition = self
@@ -496,6 +505,14 @@ class AutodiffComposition(Composition):
             autodiff_inputs = self._infer_input_nodes(inputs)
             autodiff_targets = self._infer_output_nodes(inputs)
 
+            report(self,
+                   LEARN_REPORT,
+                   # EXECUTE_REPORT,
+                   report_num=report_num,
+                   scheduler=scheduler,
+                   content='trial_start',
+                   context=context)
+
             self._build_pytorch_representation(context)
             output = self.autodiff_training(autodiff_inputs,
                                             autodiff_targets,
@@ -511,8 +528,16 @@ class AutodiffComposition(Composition):
             # FIX 5/28/20:
             context.execution_phase = execution_phase
 
+            report(self,
+                   # [LEARN_REPORT],
+                   [EXECUTE_REPORT, PROGRESS_REPORT],
+                   report_num=report_num,
+                   scheduler=scheduler,
+                   content='trial_end',
+                   context=context)
 
             scheduler.get_clock(context)._increment_time(TimeScale.TRIAL)
+
             return output
 
         return super(AutodiffComposition, self).execute(inputs=inputs,
@@ -527,7 +552,9 @@ class AutodiffComposition(Composition):
                                                         base_context=base_context,
                                                         clamp_input=clamp_input,
                                                         runtime_params=runtime_params,
-                                                        bin_execute=bin_execute,
+                                                        execution_mode=execution_mode,
+                                                        report=report,
+                                                        report_num=report_num
                                                         )
 
     def _get_state_struct_type(self, ctx):
