@@ -6,9 +6,9 @@ import psyneulink as pnl
 import psyneulink.core.llvm as pnlvm
 
 from psyneulink.core.components.component import ComponentError
-from psyneulink.core.components.functions.distributionfunctions import DriftDiffusionAnalytical, NormalDist
+from psyneulink.core.components.functions.nonstateful.distributionfunctions import DriftDiffusionAnalytical, NormalDist
 from psyneulink.core.components.functions.function import FunctionError
-from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import DriftDiffusionIntegrator
+from psyneulink.core.components.functions.stateful.integratorfunctions import DriftDiffusionIntegrator
 from psyneulink.core.components.mechanisms.processing.processingmechanism import ProcessingMechanism
 from psyneulink.core.compositions.composition import Composition
 from psyneulink.core.scheduling.condition import Never, WhenFinished
@@ -29,8 +29,8 @@ class TestReset:
         #  0.0 + 1.0 * 1.0 * 1.0 + 0.0
         D.execute(1.0)
         assert np.allclose(np.asfarray(D.value),  [[1.0], [1.0]])
-        assert np.allclose(D.output_ports[0].value[0][0], 1.0)
-        assert np.allclose(D.output_ports[1].value[0][0], 1.0)
+        assert np.allclose(D.output_ports[0].value[0], 1.0)
+        assert np.allclose(D.output_ports[1].value[0], 1.0)
 
         # reset function
         D.function.reset(2.0, 0.1)
@@ -38,8 +38,8 @@ class TestReset:
         assert np.allclose(D.function.previous_value, 2.0)
         assert np.allclose(D.function.previous_time, 0.1)
         assert np.allclose(np.asfarray(D.value),  [[1.0], [1.0]])
-        assert np.allclose(D.output_ports[0].value[0][0], 1.0)
-        assert np.allclose(D.output_ports[1].value[0][0], 1.0)
+        assert np.allclose(D.output_ports[0].value[0], 1.0)
+        assert np.allclose(D.output_ports[1].value[0], 1.0)
 
         # reset function without value spec
         D.function.reset()
@@ -47,8 +47,8 @@ class TestReset:
         assert np.allclose(D.function.previous_value, 0.0)
         assert np.allclose(D.function.previous_time, 0.0)
         assert np.allclose(np.asfarray(D.value), [[1.0], [1.0]])
-        assert np.allclose(D.output_ports[0].value[0][0], 1.0)
-        assert np.allclose(D.output_ports[1].value[0][0], 1.0)
+        assert np.allclose(D.output_ports[0].value[0], 1.0)
+        assert np.allclose(D.output_ports[1].value[0], 1.0)
 
         # reset mechanism
         D.reset(2.0, 0.1)
@@ -61,9 +61,9 @@ class TestReset:
 
         D.execute(1.0)
         #  2.0 + 1.0 = 3.0 ; 0.1 + 1.0 = 1.1
-        assert np.allclose(np.asfarray(D.value), [[[3.0]], [[1.1]]])
-        assert np.allclose(D.output_ports[0].value[0][0], 3.0)
-        assert np.allclose(D.output_ports[1].value[0][0], 1.1)
+        assert np.allclose(np.asfarray(D.value), [[3.0], [1.1]])
+        assert np.allclose(D.output_ports[0].value[0], 3.0)
+        assert np.allclose(D.output_ports[1].value[0], 1.1)
 
         # reset mechanism without value spec
         D.reset()
@@ -113,27 +113,17 @@ class TestThreshold:
         (2., [2.0, 4.0, 5.0, 5.0, 5.0]),
         (-2., [-2.0, -4.0, -5.0, -5.0, -5.0]),
         ], ids=["POSITIVE", "NEGATIVE"])
-    @pytest.mark.parametrize("mode", [
-        'Python',
-        pytest.param('LLVM', marks=pytest.mark.llvm),
-        pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-    ])
-    def test_threshold_stops_accumulation(self, mode, variable, expected, benchmark):
+    def test_threshold_stops_accumulation(self, mech_mode, variable, expected, benchmark):
         D = DDM(name='DDM',
                 function=DriftDiffusionIntegrator(threshold=5.0))
-        if mode == 'Python':
-            ex = D.execute
-        elif mode == 'LLVM':
-            ex = pnlvm.execution.MechExecution(D).execute
-        elif mode == 'PTX':
-            ex = pnlvm.execution.MechExecution(D).cuda_execute
+        ex = pytest.helpers.get_mech_execution(D, mech_mode)
 
         decision_variables = []
         time_points = []
         for i in range(5):
             output = ex([variable])
-            decision_variables.append(output[0][0][0])
-            time_points.append(output[1][0][0])
+            decision_variables.append(output[0][0])
+            time_points.append(output[1][0])
 
         # decision variable accumulation stops
         assert np.allclose(decision_variables, expected)
@@ -244,23 +234,14 @@ class TestOutputPorts:
 @pytest.mark.ddm_mechanism
 @pytest.mark.mechanism
 @pytest.mark.benchmark
-@pytest.mark.parametrize("mode", [
-    'Python',
-    pytest.param('LLVM', marks=pytest.mark.llvm),
-    pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-])
-def test_DDM_Integrator_Bogacz(benchmark, mode):
+def test_DDM_Integrator_Bogacz(benchmark, mech_mode):
     stim = 10
     T = DDM(
         name='DDM',
         function=DriftDiffusionAnalytical()
     )
-    if mode == 'Python':
-        ex = T.execute
-    elif mode == 'LLVM':
-        ex = pnlvm.execution.MechExecution(T).execute
-    elif mode == 'PTX':
-        ex = pnlvm.execution.MechExecution(T).cuda_execute
+    ex = pytest.helpers.get_mech_execution(T, mech_mode)
+
     val = ex(stim)[0]
     assert np.allclose(val, [1.0])
     if benchmark.enabled:
@@ -298,12 +279,7 @@ def test_DDM_Integrator_Bogacz(benchmark, mode):
     (0.5, 8.194383551861414),
     (2., 6.388767103722829),
     ], ids=["0", "0.5", "2.0"])
-@pytest.mark.parametrize("mode", [
-    'Python',
-    pytest.param('LLVM', marks=pytest.mark.llvm),
-    pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-])
-def test_DDM_noise(mode, benchmark, noise, expected):
+def test_DDM_noise(mech_mode, benchmark, noise, expected):
     T = DDM(
         name='DDM',
         function=DriftDiffusionIntegrator(
@@ -312,15 +288,10 @@ def test_DDM_noise(mode, benchmark, noise, expected):
             time_step_size=1.0
         )
     )
-    if mode == 'Python':
-        ex = T.execute
-    elif mode == 'LLVM':
-        ex = pnlvm.execution.MechExecution(T).execute
-    elif mode == 'PTX':
-        ex = pnlvm.execution.MechExecution(T).cuda_execute
+    ex = pytest.helpers.get_mech_execution(T, mech_mode)
 
     val = ex([10])
-    assert np.allclose(val[0][0][0], expected)
+    assert np.allclose(val[0][0], expected)
     if benchmark.enabled:
         benchmark(ex, [10])
 
@@ -432,15 +403,10 @@ def test_DDM_input_fn():
 @pytest.mark.parametrize("rate, expected", [
     (5, 50), (5., 50), ([5], 50), (-5.0, -50),
     ], ids=["int", "float", "list", "negative"])
-@pytest.mark.parametrize("mode", [
-    'Python',
-    pytest.param('LLVM', marks=pytest.mark.llvm),
-    pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-])
 # ******
 # Should negative pass?
 # ******
-def test_DDM_rate(benchmark, rate, expected, mode):
+def test_DDM_rate(benchmark, rate, expected, mech_mode):
     stim = [10]
     T = DDM(
         name='DDM',
@@ -450,13 +416,9 @@ def test_DDM_rate(benchmark, rate, expected, mode):
             time_step_size=1.0
         ),
     )
-    if mode == 'Python':
-        ex = T.execute
-    elif mode == 'LLVM':
-        ex = pnlvm.execution.MechExecution(T).execute
-    elif mode == 'PTX':
-        ex = pnlvm.execution.MechExecution(T).cuda_execute
-    val = float(ex(stim)[0][0][0])
+    ex = pytest.helpers.get_mech_execution(T, mech_mode)
+
+    val = float(ex(stim)[0][0])
     assert val == expected
     if benchmark.enabled:
         benchmark(ex, stim)
@@ -628,12 +590,12 @@ def test_DDM_time():
     np.testing.assert_allclose(time_0, 0.5, atol=1e-08)
 
     time_1 = D.execute(10)[1][0]   # t_1  = 0.5 + 0.2 = 0.7
-    np.testing.assert_allclose(time_1[0], 0.7, atol=1e-08)
+    np.testing.assert_allclose(time_1, 0.7, atol=1e-08)
 
     for i in range(10):                                     # t_11 = 0.7 + 10*0.2 = 2.7
         D.execute(10)
     time_12 = D.execute(10)[1][0]                              # t_12 = 2.7 + 0.2 = 2.9
-    np.testing.assert_allclose(time_12[0], 2.9, atol=1e-08)
+    np.testing.assert_allclose(time_12, 2.9, atol=1e-08)
 
 
 def test_WhenFinished_DDM_Analytical():
@@ -645,15 +607,7 @@ def test_WhenFinished_DDM_Analytical():
 @pytest.mark.ddm_mechanism
 @pytest.mark.mechanism
 @pytest.mark.benchmark(group="DDM-comp")
-@pytest.mark.parametrize("mode", [
-    'Python',
-    pytest.param('LLVM', marks=pytest.mark.llvm),
-    pytest.param('LLVMExec', marks=pytest.mark.llvm),
-    pytest.param('LLVMRun', marks=pytest.mark.llvm),
-    pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-    pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-])
-def test_DDM_in_composition(benchmark, mode):
+def test_DDM_in_composition(benchmark, comp_mode):
     M = pnl.DDM(
         name='DDM',
         function=pnl.DriftDiffusionIntegrator(
@@ -667,26 +621,18 @@ def test_DDM_in_composition(benchmark, mode):
     C = pnl.Composition()
     C.add_linear_processing_pathway([M])
     inputs = {M: [10]}
-    val = C.run(inputs, num_trials=2, bin_execute=mode)
+    val = C.run(inputs, num_trials=2, execution_mode=comp_mode)
     # FIXME: Python version returns dtype=object
     val = np.asfarray(val)
     assert np.allclose(val[0], [2.0])
     assert np.allclose(val[1], [0.2])
     if benchmark.enabled:
-        benchmark(C.run, inputs, num_trials=2, bin_execute=mode)
+        benchmark(C.run, inputs, num_trials=2, execution_mode=comp_mode)
 
 
 @pytest.mark.ddm_mechanism
 @pytest.mark.mechanism
-@pytest.mark.parametrize("mode", [
-    'Python',
-    pytest.param('LLVM', marks=pytest.mark.llvm),
-    pytest.param('LLVMExec', marks=pytest.mark.llvm),
-    pytest.param('LLVMRun', marks=pytest.mark.llvm),
-    pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-    pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-])
-def test_DDM_threshold_modulation(mode):
+def test_DDM_threshold_modulation(comp_mode):
     M = pnl.DDM(
         name='DDM',
         function=pnl.DriftDiffusionAnalytical(
@@ -708,34 +654,27 @@ def test_DDM_threshold_modulation(mode):
     C.add_node(monitor)
     C.add_node(control)
     inputs = {M:[1], monitor:[3]}
-    val = C.run(inputs, num_trials=1, bin_execute=mode)
+    val = C.run(inputs, num_trials=1, execution_mode=comp_mode)
     # FIXME: Python version returns dtype=object
     val = np.asfarray(val)
     assert np.allclose(val[0], [60.0])
     assert np.allclose(val[1], [60.2])
 
-@pytest.mark.parametrize("mode", ['Python',
-                                    pytest.param('LLVM', marks=pytest.mark.llvm),
-                                    pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                    pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                    pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                    pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda])
-                                    ])
 @pytest.mark.parametrize(["noise", "threshold", "expected_results"],[
                             (1.0, 0.0, (0.0, 1.0)),
                             (1.5, 2, (-2.0, 1.0)),
                             (10.0, 10.0, (10.0, 29.0)),
                             (100.0, 100.0, (100.0, 76.0)),
                         ])
-def test_ddm_is_finished(mode, noise, threshold, expected_results):
+def test_ddm_is_finished(comp_mode, noise, threshold, expected_results):
     comp = Composition()
     ddm = DDM(execute_until_finished=True,
                 function=DriftDiffusionIntegrator(threshold=threshold, noise=noise))
     comp.add_node(ddm)
 
-    results = comp.run([0], bin_execute=mode)
+    results = comp.run([0], execution_mode=comp_mode)
 
-    results = [x for x in np.array(results).flatten()] #HACK: The result is an object dtype in Python mode for some reason?
+    results = [x for x in np.array(results).flatten()] #HACK: The result is an object dtype in Python comp_mode for some reason?
     assert np.allclose(results, np.array(expected_results).flatten())
 
 
@@ -797,16 +736,10 @@ def test_sequence_of_DDM_mechs_in_Composition_Pathway():
 
 @pytest.mark.mechanism
 @pytest.mark.ddm_mechanism
-@pytest.mark.parametrize('mode', ['Python',
-                                  pytest.param('LLVM', marks=pytest.mark.llvm),
-                                  pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                  pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                  pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                  pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-def test_DDMMechanism_LCA_equivalent(mode):
+def test_DDMMechanism_LCA_equivalent(comp_mode):
     ddm = DDM(default_variable=[0], function=DriftDiffusionIntegrator(rate=1, time_step_size=0.1))
     comp2 = Composition()
     comp2.add_node(ddm)
-    result2 = comp2.run(inputs={ddm:[1]}, bin_execute=mode)
+    result2 = comp2.run(inputs={ddm:[1]}, execution_mode=comp_mode)
     assert np.allclose(np.asfarray(result2[0]), [0.1])
     assert np.allclose(np.asfarray(result2[1]), [0.1])
