@@ -6,11 +6,11 @@ import psyneulink.core.llvm as pnlvm
 
 from psyneulink.core.compositions.composition import Composition
 from psyneulink.core.components.functions.function import FunctionError
-from psyneulink.core.components.functions.distributionfunctions import NormalDist
-from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import \
+from psyneulink.core.components.functions.nonstateful.distributionfunctions import NormalDist
+from psyneulink.core.components.functions.stateful.integratorfunctions import \
     SimpleIntegrator, AdaptiveIntegrator, DriftDiffusionIntegrator, OrnsteinUhlenbeckIntegrator, \
     FitzHughNagumoIntegrator, AccumulatorIntegrator, LeakyCompetingIntegrator, DualAdaptiveIntegrator
-from psyneulink.core.components.functions.transferfunctions import Linear
+from psyneulink.core.components.functions.nonstateful.transferfunctions import Linear
 from psyneulink.core.components.mechanisms.mechanism import MechanismError
 from psyneulink.core.components.mechanisms.processing.integratormechanism import \
     IntegratorMechanism, IntegratorMechanismError
@@ -364,20 +364,6 @@ class TestReset:
 
 VECTOR_SIZE=4
 
-def _get_mechanism_execution(mech, mode):
-    if mode == 'Python':
-        def ex(variable):
-            mech.execute(variable)
-            return mech.output_values
-        return ex
-    elif mode == 'LLVM':
-        e = pnlvm.execution.MechExecution(mech)
-        return e.execute
-    elif mode == 'PTX':
-        e = pnlvm.execution.MechExecution(mech)
-        return e.cuda_execute
-    assert False, "Unknown execution mode: {}".format(mode)
-
 class TestIntegratorFunctions:
 
     @pytest.mark.mechanism
@@ -398,16 +384,13 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_integrator_multiple_input(self, benchmark, mode):
+    def test_integrator_multiple_input(self, benchmark, mech_mode):
         I = IntegratorMechanism(
             function=Linear(slope=2.0, intercept=1.0),
             default_variable=[[1], [2]],
             input_ports=['a', 'b'],
         )
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = ex([[1], [2]])
         assert np.allclose(val, [[3]])
@@ -418,15 +401,12 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_integrator_multiple_output(self, benchmark, mode):
+    def test_integrator_multiple_output(self, benchmark, mech_mode):
         I = IntegratorMechanism(
             default_variable=[5],
             output_ports=[{pnl.VARIABLE: (pnl.OWNER_VALUE, 0)}, 'c'],
         )
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = ex([5])
         assert np.allclose(val, [[2.5], [2.5]])
@@ -437,10 +417,7 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_integrator_multiple_input_output(self, benchmark, mode):
+    def test_integrator_multiple_input_output(self, benchmark, mech_mode):
         I = IntegratorMechanism(
             function=Linear(slope=2.0, intercept=1.0),
             default_variable=[[1], [2]],
@@ -448,7 +425,7 @@ class TestIntegratorFunctions:
             output_ports=[{pnl.VARIABLE: (pnl.OWNER_VALUE, 1)},
                           {pnl.VARIABLE: (pnl.OWNER_VALUE, 0)}],
         )
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = ex([[1], [2]])
         assert np.allclose(val, [[5], [3]])
@@ -458,15 +435,12 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_FitzHughNagumo_simple_scalar(self, benchmark, mode):
+    def test_FitzHughNagumo_simple_scalar(self, benchmark, mech_mode):
         var = [1.0]
         I = IntegratorMechanism(name="I",
                                 default_variable=[var],
                                 function=FitzHughNagumoIntegrator())
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = ex(var)
         assert np.allclose(val[0], [0.05127053])
@@ -476,15 +450,12 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_FitzHughNagumo_simple_vector(self, benchmark, mode):
+    def test_FitzHughNagumo_simple_vector(self, benchmark, mech_mode):
         var = [1.0, 3.0]
         I = IntegratorMechanism(name="I",
                                 default_variable=var,
                                 function=FitzHughNagumoIntegrator)
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = ex(var)
         assert np.allclose(val[0], [0.05127053, 0.15379818])
@@ -494,14 +465,11 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_integrator(self, benchmark, mode):
+    def test_transfer_integrator(self, benchmark, mech_mode):
         I = IntegratorMechanism(
             default_variable=[0 for i in range(VECTOR_SIZE)],
             function=Linear(slope=5.0))
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = benchmark(ex, [1.0 for i in range(VECTOR_SIZE)])
         assert np.allclose(val, [[5.0 for i in range(VECTOR_SIZE)]])
@@ -562,7 +530,7 @@ class TestIntegratorFunctions:
         # P = Process(pathway=[I])
         # 10 + 10*0.5 + 0 + 10 = 25
         val = I.execute(1)
-        assert np.allclose([[[25.]], [[0.5]]], val)
+        assert np.allclose([[25.], [0.5]], val)
 
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
@@ -630,12 +598,9 @@ class TestIntegratorFunctions:
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
     @pytest.mark.benchmark(group="IntegratorMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_integrator_no_function(self, benchmark, mode):
+    def test_integrator_no_function(self, benchmark, mech_mode):
         I = IntegratorMechanism()
-        ex = _get_mechanism_execution(I, mode)
+        ex = pytest.helpers.get_mech_execution(I, mech_mode)
 
         val = ex([10])
         assert np.allclose(val, [[5.0]])
@@ -798,7 +763,7 @@ class TestIntegratorRate:
         )
         # P = Process(pathway=[I])
         val = I.execute(10.0)
-        assert np.allclose([[[50.0]], [[1.0]]], val)
+        assert np.allclose([[50.0], [1.0]], val)
 
     # rate = list, integration_type = simple
 
@@ -985,7 +950,7 @@ class TestIntegratorRate:
                 ))
 
         error_msg_a = "The parameters with len>1 specified for AccumulatorIntegrator Function-0 "
-        error_msg_b = "(['rate', 'increment']) don't all have the same length"
+        error_msg_b = "(['increment', 'rate']) don't all have the same length"
         assert error_msg_a in str(error_text.value)
         assert error_msg_b in str(error_text.value)
 
@@ -1178,7 +1143,7 @@ class TestIntegratorNoise:
         )
 
         val = I.execute(10.0)
-        assert np.allclose(val, [[[4.29013944]], [[ 1.        ]]])
+        assert np.allclose(val, [[4.29013944], [ 1.        ]])
 
 # COMMENTED OUT UNTIL OU INTEGRATOR IS VALIDATED
     @pytest.mark.mechanism
@@ -1210,11 +1175,6 @@ class TestStatefulness:
 
     @pytest.mark.mechanism
     @pytest.mark.integrator_mechanism
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                      pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                      pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                      pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda])])
     @pytest.mark.parametrize('cond0, cond1, expected', [
         (pnl.Never(), pnl.AtTrial(2),
          [[np.array([0.5]), np.array([0.5])],
@@ -1241,7 +1201,10 @@ class TestStatefulness:
           [np.array([0.5]), np.array([0.9375])],
           [np.array([0.5]), np.array([0.96875])]]),
         ], ids=lambda x: str(x) if isinstance(x, pnl.Condition) else "")
-    def test_reset_stateful_function_when_composition(self, mode, cond0, cond1, expected):
+    # 'LLVM' mode is not supported, because synchronization of compiler and
+    # python values during execution is not implemented.
+    @pytest.mark.usefixtures("comp_mode_no_llvm")
+    def test_reset_stateful_function_when_composition(self, comp_mode, cond0, cond1, expected):
         I1 = pnl.IntegratorMechanism()
         I2 = pnl.IntegratorMechanism()
         I1.reset_stateful_function_when = cond0
@@ -1250,7 +1213,7 @@ class TestStatefulness:
         C.add_node(I1)
         C.add_node(I2)
 
-        C.run(inputs={I1: [[1.0]], I2: [[1.0]]}, num_trials=7, bin_execute=mode)
+        C.run(inputs={I1: [[1.0]], I2: [[1.0]]}, num_trials=7, execution_mode=comp_mode)
 
         assert np.allclose(expected, C.results)
 
