@@ -3,12 +3,12 @@ import pytest
 
 import psyneulink.core.llvm as pnlvm
 from psyneulink.core.components.component import ComponentError
-from psyneulink.core.components.functions.learningfunctions import Reinforcement
-from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import AccumulatorIntegrator, AdaptiveIntegrator
-from psyneulink.core.components.functions.transferfunctions import Linear, Exponential, Logistic, ReLU, SoftMax
-from psyneulink.core.components.functions.combinationfunctions import Reduce
+from psyneulink.core.components.functions.nonstateful.learningfunctions import Reinforcement
+from psyneulink.core.components.functions.stateful.integratorfunctions import AccumulatorIntegrator, AdaptiveIntegrator
+from psyneulink.core.components.functions.nonstateful.transferfunctions import Linear, Exponential, Logistic, ReLU, SoftMax
+from psyneulink.core.components.functions.nonstateful.combinationfunctions import Reduce
 from psyneulink.core.components.functions.userdefinedfunction import UserDefinedFunction
-from psyneulink.core.components.functions.distributionfunctions import NormalDist, UniformToNormalDist, \
+from psyneulink.core.components.functions.nonstateful.distributionfunctions import NormalDist, UniformToNormalDist, \
     ExponentialDist, \
     UniformDist, GammaDist, WaldDist
 from psyneulink.core.components.functions.function import FunctionError
@@ -19,8 +19,7 @@ from psyneulink.library.components.mechanisms.processing.transfer.lcamechanism i
 from psyneulink.core.components.ports.inputport import InputPort
 from psyneulink.core.compositions.composition import Composition
 from psyneulink.core.globals.keywords import \
-    INSTANTANEOUS_MODE_VALUE, INTEGRATOR_MODE_VALUE, RESET, COMBINE, RESULT, GREATER_THAN
-from psyneulink.core.globals.utilities import UtilitiesError, set_global_seed
+    CURRENT_VALUE, LAST_INTEGRATED_VALUE, RESET, COMBINE, GREATER_THAN
 from psyneulink.core.globals.parameters import ParameterError
 from psyneulink.core.scheduling.condition import Never
 from psyneulink.core.scheduling.time import TimeScale
@@ -50,10 +49,7 @@ class TestTransferMechanismInputs:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_inputs_list_of_floats(self, benchmark, mode):
+    def test_transfer_mech_inputs_list_of_floats(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -63,14 +59,7 @@ class TestTransferMechanismInputs:
         )
         T.reset_stateful_function_when = Never()
         var = [10.0 for i in range(VECTOR_SIZE)]
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         val = EX(var)
         assert np.allclose(val, [[10.0 for i in range(VECTOR_SIZE)]])
@@ -151,10 +140,7 @@ class TestTransferMechanismNoise:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Linear noise")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_var_float_noise(self, benchmark, mode):
+    def test_transfer_mech_array_var_float_noise(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -165,14 +151,7 @@ class TestTransferMechanismNoise:
             integrator_mode=True
         )
         T.reset_stateful_function_when = Never()
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [0 for i in range(VECTOR_SIZE)]
         val = EX(var)
@@ -216,10 +195,7 @@ class TestTransferMechanismNoise:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Linear noise2")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_var_normal_array_noise2(self, benchmark, mode):
+    def test_transfer_mech_array_var_normal_array_noise2(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -230,14 +206,7 @@ class TestTransferMechanismNoise:
             integrator_mode=True
         )
         T.reset_stateful_function_when = Never()
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [0 for i in range(VECTOR_SIZE)]
         val = EX(var)
@@ -438,10 +407,7 @@ class TestTransferMechanismFunctions:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Logistic")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_logistic_fun(self, benchmark, mode):
+    def test_transfer_mech_logistic_fun(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -450,14 +416,7 @@ class TestTransferMechanismFunctions:
             integration_rate=1.0,
             integrator_mode=True
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [0 for i in range(VECTOR_SIZE)]
         val = EX(var)
@@ -468,10 +427,7 @@ class TestTransferMechanismFunctions:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism ReLU")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_relu_fun(self, benchmark, mode):
+    def test_transfer_mech_relu_fun(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -480,14 +436,7 @@ class TestTransferMechanismFunctions:
             integration_rate=1.0,
             integrator_mode=True
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         val1 = EX([0 for i in range(VECTOR_SIZE)])
         val2 = EX([1 for i in range(VECTOR_SIZE)])
@@ -503,10 +452,7 @@ class TestTransferMechanismFunctions:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Exponential")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_exponential_fun(self, benchmark, mode):
+    def test_transfer_mech_exponential_fun(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -515,14 +461,7 @@ class TestTransferMechanismFunctions:
             integration_rate=1.0,
             integrator_mode=True
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [0 for i in range(VECTOR_SIZE)]
         val = EX(var)
@@ -533,10 +472,7 @@ class TestTransferMechanismFunctions:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism SoftMax")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_softmax_fun(self, benchmark, mode):
+    def test_transfer_mech_softmax_fun(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -545,14 +481,7 @@ class TestTransferMechanismFunctions:
             integration_rate=1.0,
             integrator_mode=True
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [0 for i in range(VECTOR_SIZE)]
         val = EX(var)
@@ -630,10 +559,7 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_mech_rate(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_mech_rate(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -642,14 +568,7 @@ class TestTransferMechanismIntegratorFunctionParams:
             integrator_function=AdaptiveIntegrator,
             integration_rate=[i / 10 for i in range(VECTOR_SIZE)]
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -661,10 +580,7 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_fct_rate(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_fct_rate(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -672,14 +588,7 @@ class TestTransferMechanismIntegratorFunctionParams:
             integrator_mode=True,
             integrator_function=AdaptiveIntegrator(rate=[i / 10 for i in range(VECTOR_SIZE)])
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -691,10 +600,7 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_fct_over_mech_rate(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_fct_over_mech_rate(self, benchmark, mech_mode):
 
         T = TransferMechanism(
                 name='T',
@@ -703,14 +609,7 @@ class TestTransferMechanismIntegratorFunctionParams:
                 integrator_function=AdaptiveIntegrator(rate=[i / 20 for i in range(VECTOR_SIZE)]),
                 integration_rate=[i / 10 for i in range(VECTOR_SIZE)]
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -752,24 +651,14 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_mech_init_val(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_mech_init_val(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0 for i in range(VECTOR_SIZE)],
             integrator_mode=True,
             initial_value=[i / 10 for i in range(VECTOR_SIZE)]
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -782,10 +671,7 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_fct_initzr(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_fct_initzr(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0 for i in range(VECTOR_SIZE)],
@@ -795,15 +681,7 @@ class TestTransferMechanismIntegratorFunctionParams:
                     initializer=[i / 10 for i in range(VECTOR_SIZE)]
             ),
         )
-        if mode == 'Python':
-            EX = T.execute
-            assert True
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -816,10 +694,7 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_fct_initlzr_over_mech_init_val(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_fct_initlzr_over_mech_init_val(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0 for i in range(VECTOR_SIZE)],
@@ -830,14 +705,7 @@ class TestTransferMechanismIntegratorFunctionParams:
             ),
             initial_value=[i / 10 for i in range(VECTOR_SIZE)]
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -920,10 +788,7 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_array_assignments_mech_noise(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_mech_noise(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -932,14 +797,7 @@ class TestTransferMechanismIntegratorFunctionParams:
             integrator_function=AdaptiveIntegrator,
             noise=[i / 10 for i in range(VECTOR_SIZE)]
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -951,11 +809,8 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=[pytest.mark.llvm, pytest.mark.skip]),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda, pytest.mark.skip])])
     # FIXME: Incorrect T.integrator_function.defaults.variable reported
-    def test_transfer_mech_array_assignments_fct_noise(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_fct_noise(self, benchmark, mech_mode):
 
         T = TransferMechanism(
             name='T',
@@ -963,14 +818,7 @@ class TestTransferMechanismIntegratorFunctionParams:
             integrator_mode=True,
             integrator_function=AdaptiveIntegrator(noise=[i / 10 for i in range(VECTOR_SIZE)])
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -982,11 +830,8 @@ class TestTransferMechanismIntegratorFunctionParams:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Parameter Array Assignments")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=[pytest.mark.llvm, pytest.mark.skip]),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda, pytest.mark.skip])])
     # FIXME: Incorrect T.integrator_function.defaults.variable reported
-    def test_transfer_mech_array_assignments_fct_over_mech_noise(self, benchmark, mode):
+    def test_transfer_mech_array_assignments_fct_over_mech_noise(self, benchmark, mech_mode):
 
         T = TransferMechanism(
                 name='T',
@@ -995,14 +840,7 @@ class TestTransferMechanismIntegratorFunctionParams:
                 integrator_function=AdaptiveIntegrator(noise=[i / 20 for i in range(VECTOR_SIZE)]),
                 noise=[i / 10 for i in range(VECTOR_SIZE)]
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         var = [1 for i in range(VECTOR_SIZE)]
         EX(var)
@@ -1023,8 +861,9 @@ class TestTransferMechanismIntegratorFunctionParams:
             )
         assert (
             "Noise parameter ([0.0, 0.1, 0.2, 0.3, 0.4])" in str(error_text.value) and
-            "does not match default variable ([[0 0 0 0]])." in str(error_text.value) and
-            "must be specified as a float, a function, or an array of the appropriate shape ((1, 4))." in str(error_text.value)
+            "does not match default variable ([[0 0 0 0]]);" in str(error_text.value) and
+            "must be specified as a float, a function, or an array of the appropriate shape ((1, 4))."
+            in str(error_text.value)
         )
 
     # def test_transfer_mech_array_assignments_wrong_size_fct_noise(self, benchmark, mode):
@@ -1049,10 +888,7 @@ class TestTransferMechanismTimeConstant:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Linear TimeConstant=1")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_integration_rate_0_8(self, benchmark, mode):
+    def test_transfer_mech_integration_rate_0_8(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0 for i in range(VECTOR_SIZE)],
@@ -1060,14 +896,7 @@ class TestTransferMechanismTimeConstant:
             integration_rate=0.8,
             integrator_mode=True
         )
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.execute
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            EX = e.cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         val1 = T.execute([1 for i in range(VECTOR_SIZE)])
         val2 = T.execute([1 for i in range(VECTOR_SIZE)])
@@ -1081,10 +910,7 @@ class TestTransferMechanismTimeConstant:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Linear TimeConstant=1")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_smoothin_factor_1_0(self, benchmark, mode):
+    def test_transfer_mech_smoothin_factor_1_0(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0 for i in range(VECTOR_SIZE)],
@@ -1092,24 +918,15 @@ class TestTransferMechanismTimeConstant:
             integration_rate=1.0,
             integrator_mode=True
         )
-        if mode == 'Python':
-            val = benchmark(T.execute, [1 for i in range(VECTOR_SIZE)])
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.execute, [1 for i in range(VECTOR_SIZE)])
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.cuda_execute, [1 for i in range(VECTOR_SIZE)])
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
+        val = benchmark(EX, [1 for i in range(VECTOR_SIZE)])
         assert np.allclose(val, [[1.0 for i in range(VECTOR_SIZE)]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism Linear TimeConstant=0")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_integration_rate_0_0(self, benchmark, mode):
+    def test_transfer_mech_integration_rate_0_0(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0 for i in range(VECTOR_SIZE)],
@@ -1117,21 +934,14 @@ class TestTransferMechanismTimeConstant:
             integration_rate=0.0,
             integrator_mode=True
         )
-        if mode == 'Python':
-            val = benchmark(T.execute, [1 for i in range(VECTOR_SIZE)])
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.execute, [1 for i in range(VECTOR_SIZE)])
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.cuda_execute, [1 for i in range(VECTOR_SIZE)])
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
+
+        val = benchmark(EX, [1 for i in range(VECTOR_SIZE)])
         assert np.allclose(val, [[0.0 for i in range(VECTOR_SIZE)]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=[pytest.mark.llvm, pytest.mark.skip])])
-    def test_transfer_mech_integration_rate_0_8_initial_0_5(self, mode):
+    def test_transfer_mech_integration_rate_0_8_initial_0_5(self, mech_mode):
         T = TransferMechanism(
             name='T',
             default_variable=[0, 0, 0, 0],
@@ -1140,20 +950,24 @@ class TestTransferMechanismTimeConstant:
             initial_value=np.array([[.5, .5, .5, .5]]),
             integrator_mode=True
         )
-        if mode == 'Python':
-            val = T.execute([1, 1, 1, 1])
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            val = e.execute([1, 1, 1, 1])
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
+
+        val = EX([1, 1, 1, 1])
         assert np.allclose(val, [[0.9, 0.9, 0.9, 0.9]])
 
+        # FIXME: The code bellow modifies parameter value.
+        #        This is not support in compiled mode.
+        if mech_mode != 'Python':
+            return
+
         T.noise.base = 10
 
-        if mode == 'Python':
+        if mech_mode == 'Python':
             val = T.execute([1, 2, -3, 0])
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
+        elif mech_mode == 'LLVM':
             val = e.execute([1, 2, -3, 0])
+        elif mech_mode == 'PTX':
+            val = e.cuda_execute([1, 2, -3, 0])
         assert np.allclose(val, [[10.98, 11.78, 7.779999999999999, 10.18]]) # testing noise changes to an integrator
 
     # @pytest.mark.mechanism
@@ -1534,23 +1348,15 @@ class TestTransferMechanismMultipleInputPorts:
     @pytest.mark.transfer_mechanism
     @pytest.mark.mimo
     @pytest.mark.benchmark(group="MIMO")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_transfer_mech_2d_variable(self, benchmark, mode):
+    def test_transfer_mech_2d_variable(self, benchmark, mech_mode):
         T = TransferMechanism(
             name='T',
             function=Linear(slope=2.0, intercept=1.0),
             default_variable=[[0.0, 0.0], [0.0, 0.0]],
         )
-        if mode == 'Python':
-            val = benchmark(T.execute, [[1.0, 2.0], [3.0, 4.0]])
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.execute, [[1.0, 2.0], [3.0, 4.0]])
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.cuda_execute, [[1.0, 2.0], [3.0, 4.0]])
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
+
+        val = benchmark(EX, [[1.0, 2.0], [3.0, 4.0]])
         assert np.allclose(val, [[3., 5.], [7., 9.]])
 
     @pytest.mark.mechanism
@@ -1569,24 +1375,14 @@ class TestTransferMechanismMultipleInputPorts:
     @pytest.mark.transfer_mechanism
     @pytest.mark.mimo
     @pytest.mark.benchmark(group="MIMO")
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_multiple_output_ports_for_multiple_input_ports(self, benchmark, mode):
+    def test_multiple_output_ports_for_multiple_input_ports(self, benchmark, mech_mode):
         T = TransferMechanism(input_ports=['a','b','c'])
-        if mode == 'Python':
-            val = benchmark(T.execute, [[1], [2], [3]])
-            assert all(a==b for a,b in zip(T.output_values,val))
-        elif mode == 'LLVM':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.execute, [[1], [2], [3]])
-        elif mode == 'PTX':
-            e = pnlvm.execution.MechExecution(T)
-            val = benchmark(e.cuda_execute, [[1], [2], [3]])
+        assert len(T.variable) == 3
+        assert len(T.output_ports) == 3
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
-        assert len(T.variable)==3
-        assert all(a==b for a,b in zip(val, [[ 1.],[ 2.],[ 3.]]))
-        assert len(T.output_ports)==3
+        val = benchmark(EX, [[1], [2], [3]])
+        assert all(a == b for a,b in zip(val, [[ 1.],[ 2.],[ 3.]]))
 
     # @pytest.mark.mechanism
     # @pytest.mark.transfer_mechanism
@@ -1827,12 +1623,13 @@ class TestIntegratorMode:
             T_not_integrator = TransferMechanism()
             T_not_integrator.execute(1.0)
             T_not_integrator.reset(0.0)
-        assert "not allowed because this Mechanism is not stateful;" in str(err_txt.value)
-        assert "try setting the integrator_mode argument to True." in str(err_txt.value)
+
+        assert "not allowed because its `integrator_mode` parameter" in str(err_txt.value)
+        assert "is currently set to \'False\'; try setting it to \'True\'" in str(err_txt.value)
 
     def test_switch_mode(self):
         T = TransferMechanism(integrator_mode=True,
-                              on_resume_integrator_mode=INTEGRATOR_MODE_VALUE)
+                              on_resume_integrator_mode=LAST_INTEGRATED_VALUE)
         C = Composition(pathways=[T])
         integrator_function = T.integrator_function
         T.reset_stateful_function_when = Never()
@@ -1898,8 +1695,8 @@ class TestIntegratorMode:
 
 class TestOnResumeIntegratorMode:
 
-    def test_integrator_mode_value_spec(self):
-        T = TransferMechanism(on_resume_integrator_mode=INTEGRATOR_MODE_VALUE,
+    def test_last_integrated_value_spec(self):
+        T = TransferMechanism(on_resume_integrator_mode=LAST_INTEGRATED_VALUE,
                               integration_rate=0.5,
                               integrator_mode=True)
         C = Composition()
@@ -1924,8 +1721,8 @@ class TestOnResumeIntegratorMode:
         # Trial 1: 0.5*1.125 + 0.5*2.0 = 1.5625 * 1.0 = 1.5625
         assert np.allclose(T.parameters.value.get(C), [[1.5625]])
 
-    def test_instantaneous_mode_value_spec(self):
-        T = TransferMechanism(on_resume_integrator_mode=INSTANTANEOUS_MODE_VALUE,
+    def test_current_value_spec(self):
+        T = TransferMechanism(on_resume_integrator_mode=CURRENT_VALUE,
                               integration_rate=0.5,
                               integrator_mode=True)
         C = Composition()
@@ -1981,17 +1778,10 @@ class TestOnResumeIntegratorMode:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     @pytest.mark.benchmark(group="TransferMechanism")
-    @pytest.mark.parametrize('bin_execute', ['Python',
-                                             # 'LLVM' mode is not supported
-                                             # the comparison values and checks
-                                             # are not synced between binary
-                                             # and Python structures
-                                             pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                             pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                             pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                             pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                            ])
-    def test_termination_measures(self, bin_execute):
+    # 'LLVM' mode is not supported, because synchronization of compiler and
+    # python values during execution is not implemented.
+    @pytest.mark.usefixtures("comp_mode_no_llvm")
+    def test_termination_measures(self, comp_mode):
         stim_input = ProcessingMechanism(size=2, name='Stim Input')
         stim_percept = TransferMechanism(name='Stimulus', size=2, function=Logistic)
         instruction_input = ProcessingMechanism(size=2, function=Linear(slope=10))
@@ -2013,10 +1803,10 @@ class TestOnResumeIntegratorMode:
         comp.add_linear_processing_pathway([instruction_input, attention, stim_percept])
         inputs = {stim_input: [[1, 1], [1, 1]],
                   instruction_input: [[1, -1], [-1, 1]]}
-        result = comp.run(inputs=inputs, bin_execute=bin_execute)
+        result = comp.run(inputs=inputs, execution_mode=comp_mode)
 
         assert np.allclose(result, [[0.43636140750487973, 0.47074475219780554]])
-        if bin_execute == 'Python':
+        if comp_mode is pnlvm.ExecutionMode.Python:
             assert decision.num_executions.time_step == 1
             assert decision.num_executions.pass_ == 2
             assert decision.num_executions.trial== 1
@@ -2026,17 +1816,9 @@ class TestOnResumeIntegratorMode:
 class TestClip:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_clip_float(self, mode):
+    def test_clip_float(self, mech_mode):
         T = TransferMechanism(clip=[-2.0, 2.0])
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            EX = pnlvm.execution.MechExecution(T).execute
-        elif mode == 'PTX':
-            EX = pnlvm.execution.MechExecution(T).cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         assert np.allclose(EX(3.0), 2.0)
         assert np.allclose(EX(1.0), 1.0)
@@ -2044,34 +1826,19 @@ class TestClip:
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_clip_array(self, mode):
+    def test_clip_array(self, mech_mode):
         T = TransferMechanism(default_variable=[[0.0, 0.0, 0.0]],
                               clip=[-2.0, 2.0])
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            EX = pnlvm.execution.MechExecution(T).execute
-        elif mode == 'PTX':
-            EX = pnlvm.execution.MechExecution(T).cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
+
         assert np.allclose(EX([3.0, 0.0, -3.0]), [2.0, 0.0, -2.0])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
-    @pytest.mark.parametrize('mode', ['Python',
-                                      pytest.param('LLVM', marks=pytest.mark.llvm),
-                                      pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-    def test_clip_2d_array(self, mode):
+    def test_clip_2d_array(self, mech_mode):
         T = TransferMechanism(default_variable=[[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]],
                               clip=[-2.0, 2.0])
-        if mode == 'Python':
-            EX = T.execute
-        elif mode == 'LLVM':
-            EX = pnlvm.execution.MechExecution(T).execute
-        elif mode == 'PTX':
-            EX = pnlvm.execution.MechExecution(T).cuda_execute
+        EX = pytest.helpers.get_mech_execution(T, mech_mode)
 
         assert np.allclose(EX([[-5.0, -1.0, 5.0], [5.0, -5.0, 1.0], [1.0, 5.0, 5.0]]),
                            [[-2.0, -1.0, 2.0], [2.0, -2.0, 1.0], [1.0, 2.0, 2.0]])
