@@ -14,8 +14,8 @@ import psyneulink as pnl
 # Note that this script implements a slightly different Figure than in the original Figure in the paper.
 # However, this implementation is identical with a plot we created with an old MATLAB code which was used for the
 # conflict monitoring simulations.
-import psyneulink.core.components.functions.objectivefunctions
-import psyneulink.core.components.functions.transferfunctions
+import psyneulink.core.components.functions.nonstateful.objectivefunctions
+import psyneulink.core.components.functions.nonstateful.transferfunctions
 
 
 @pytest.mark.model
@@ -23,13 +23,7 @@ import psyneulink.core.components.functions.transferfunctions
 @pytest.mark.parametrize("reps", [1,
                                   pytest.param(10, marks=pytest.mark.stress),
                                   pytest.param(100, marks=pytest.mark.stress)])
-@pytest.mark.parametrize("mode", ['Python',
-                                  pytest.param('LLVM', marks=pytest.mark.llvm),
-                                  pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                  pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                  pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                  pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-def test_botvinick_model(benchmark, mode, reps):
+def test_botvinick_model(benchmark, comp_mode, reps):
     benchmark.group = "Botvinick (scale " + str(reps / 100) + ")"
 
     # SET UP MECHANISMS ----------------------------------------------------------------------------------------------------
@@ -185,20 +179,20 @@ def test_botvinick_model(benchmark, mode, reps):
     ntrials0 = 5 * reps
     ntrials = 10 * reps
 
-    def run(bin_execute):
+    def run(mode):
         results = []
         for i, stim in enumerate(Stimulus):
             # RUN the COMPOSITION to initialize --------------------------------
             exec_id = "exec_" + str(i)
-            comp.run(inputs=stim[0], num_trials=ntrials0, bin_execute=bin_execute, context=exec_id)
-            comp.run(inputs=stim[1], num_trials=ntrials, bin_execute=bin_execute, context=exec_id)
+            comp.run(inputs=stim[0], num_trials=ntrials0, execution_mode=mode, context=exec_id)
+            comp.run(inputs=stim[1], num_trials=ntrials, execution_mode=mode, context=exec_id)
 
             # Comp results include concatenation of both the above runs
             results.append(comp.results)
 
         return results
 
-    res = run(mode)
+    res = run(comp_mode)
     # the corresponding output port indices in composition results
     # these were 0 and 1 in the prior version of the test
     response_results_index = 3
@@ -290,4 +284,4 @@ def test_botvinick_model(benchmark, mode, reps):
         assert np.allclose(res[2][ntrials0 - 1][response_decision_energy_index], [0.94440397])
         assert np.allclose(res[2][-1][response_decision_energy_index], [0.90033387])
     if benchmark.enabled:
-        benchmark(run, mode)
+        benchmark(run, comp_mode)
