@@ -1,28 +1,24 @@
-import pytest
 import numpy as np
+import pytest
 
+import psyneulink as pnl
+from psyneulink.core.components.functions.nonstateful.objectivefunctions import Distance
+from psyneulink.core.components.functions.nonstateful.optimizationfunctions import GridSearch, MINIMIZE
+from psyneulink.core.components.functions.nonstateful.transferfunctions import GaussianDistort
+from psyneulink.core.components.mechanisms.modulatory.control import OptimizationControlMechanism
 from psyneulink.core.components.mechanisms.processing.objectivemechanism import ObjectiveMechanism
 from psyneulink.core.components.mechanisms.processing.processingmechanism import ProcessingMechanism
 from psyneulink.core.components.mechanisms.processing.transfermechanism import TransferMechanism
-from psyneulink.core.components.mechanisms.modulatory.control import OptimizationControlMechanism
-from psyneulink.library.components.mechanisms.processing.objective.comparatormechanism import ComparatorMechanism
-from psyneulink.core.components.functions.objectivefunctions import Distance
-from psyneulink.core.components.functions.optimizationfunctions import GridSearch, MINIMIZE
-from psyneulink.core.components.functions.transferfunctions import GaussianDistort, Exponential
-from psyneulink.core.components.ports.modulatorysignals.controlsignal import ControlSignal
 from psyneulink.core.components.ports.inputport import SHADOW_INPUTS
+from psyneulink.core.components.ports.modulatorysignals.controlsignal import ControlSignal
 from psyneulink.core.compositions.composition import Composition, NodeRole
 from psyneulink.core.globals.keywords import VARIANCE, NORMED_L0_SIMILARITY
+from psyneulink.library.components.mechanisms.processing.objective.comparatormechanism import ComparatorMechanism
+
 
 @pytest.mark.model
 @pytest.mark.benchmark(group="Greedy Agent")
-@pytest.mark.parametrize("mode", ['Python',
-    pytest.param('LLVM', marks=[pytest.mark.llvm]),
-    pytest.param('LLVMExec', marks=[pytest.mark.llvm]),
-    pytest.param('LLVMRun', marks=[pytest.mark.llvm]),
-    pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-    pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-def test_simplified_greedy_agent(benchmark, mode):
+def test_simplified_greedy_agent(benchmark, comp_mode):
     # These should probably be replaced by reference to ForagerEnv constants:
     obs_len = 2
     action_len = 2
@@ -57,23 +53,17 @@ def test_simplified_greedy_agent(benchmark, mode):
 
     run_results = agent_comp.run(inputs={player:[[619,177]],
                                          prey:[[419,69]]},
-                                 bin_execute=mode)
+                                 execution_mode=comp_mode)
     assert np.allclose(run_results, [[-200, -108]])
     if benchmark.enabled:
         benchmark(agent_comp.run, **{'inputs':{
             player:[[619,177]],
             prey:[[419,69]],
-            }, 'bin_execute':mode})
+            }, 'execution_mode':comp_mode})
 
 @pytest.mark.model
 @pytest.mark.benchmark(group="Greedy Agant Random")
-@pytest.mark.parametrize("mode", ['Python',
-    pytest.param('LLVM', marks=[pytest.mark.llvm]),
-    pytest.param('LLVMExec', marks=[pytest.mark.llvm]),
-    pytest.param('LLVMRun', marks=[pytest.mark.llvm]),
-    pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-    pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda])])
-def test_simplified_greedy_agent_random(benchmark, mode):
+def test_simplified_greedy_agent_random(benchmark, comp_mode):
     # These should probably be replaced by reference to ForagerEnv constants:
     obs_len = 2
     action_len = 2
@@ -105,7 +95,7 @@ def test_simplified_greedy_agent_random(benchmark, mode):
 
     run_results = agent_comp.run(inputs={player:[[619,177]],
                                          prey:[[419,69]]},
-                                 bin_execute=mode)
+                                 execution_mode=comp_mode)
     # KDM 12/4/19: modified results due to global seed offset of
     # GaussianDistort assignment.
     # to produce old numbers, run get_global_seed once before creating
@@ -115,28 +105,31 @@ def test_simplified_greedy_agent_random(benchmark, mode):
         benchmark(agent_comp.run, **{'inputs':{
             player:[[619,177]],
             prey:[[419,69]],
-            }, 'bin_execute':mode})
+            }, 'execution_mode':comp_mode})
 
 @pytest.mark.model
 @pytest.mark.benchmark(group="Predator Prey")
-@pytest.mark.parametrize("mode", ['Python',
-     pytest.param('Python-PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-     pytest.param('LLVM', marks=[pytest.mark.llvm]),
-     pytest.param('LLVMExec', marks=[pytest.mark.llvm]),
-     pytest.param('LLVMRun', marks=[pytest.mark.llvm]),
-     pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-     pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-])
+@pytest.mark.parametrize("mode", pytest.helpers.get_comp_execution_modes() +
+                                 [pytest.helpers.cuda_param('Python-PTX'),
+                                  pytest.param('Python-LLVM', marks=pytest.mark.llvm)])
 @pytest.mark.parametrize("samples", [[0,10],
     pytest.param([0,3,6,10], marks=pytest.mark.stress),
     pytest.param([0,2,4,6,8,10], marks=pytest.mark.stress),
     pytest.param([a / 10.0 for a in range(0, 101)]),
 ], ids=lambda x: len(x))
 def test_predator_prey(benchmark, mode, samples):
-    if len(samples) > 10 and mode not in {"LLVM", "LLVMRun", "Python-PTX"}:
+    if len(samples) > 10 and mode not in {pnl.ExecutionMode.LLVM,
+                                          pnl.ExecutionMode.LLVMExec,
+                                          pnl.ExecutionMode.LLVMRun,
+                                          "Python-PTX", "Python-LLVM"}:
         pytest.skip("This test takes too long")
-    # OCM default mode is Python
-    mode, ocm_mode = (mode + "-Python").split('-')[0:2]
+    if str(mode).startswith('Python-'):
+        ocm_mode = mode.split('-')[1]
+        mode = pnl.ExecutionMode.Python
+    else:
+        # OCM default mode is Python
+        ocm_mode = 'Python'
+
     benchmark.group = "Predator-Prey " + str(len(samples))
     obs_len = 3
     obs_coords = 2
@@ -173,7 +166,7 @@ def test_predator_prey(benchmark, mode, samples):
     agent_comp.add_node(greedy_action_mech)
     agent_comp.exclude_node_roles(predator_obs, NodeRole.OUTPUT)
 
-    ocm = OptimizationControlMechanism(features={SHADOW_INPUTS: [player_obs, predator_obs, prey_obs]},
+    ocm = OptimizationControlMechanism(state_features={SHADOW_INPUTS: [player_obs, predator_obs, prey_obs]},
                                        agent_rep=agent_comp,
                                        function=GridSearch(direction=MINIMIZE,
                                                            save_values=True),
@@ -199,7 +192,7 @@ def test_predator_prey(benchmark, mode, samples):
                   predator_obs:[[-0.03479106, -0.47666293]],
                   prey_obs:[[-0.60836214,  0.1760381 ]],
                  }
-    run_results = agent_comp.run(inputs=input_dict, num_trials=2, bin_execute=mode)
+    run_results = agent_comp.run(inputs=input_dict, num_trials=2, execution_mode=mode)
 
     if len(samples) == 2:
         # KDM 12/4/19: modified results due to global seed offset of
@@ -207,10 +200,10 @@ def test_predator_prey(benchmark, mode, samples):
         # to produce old numbers, run get_global_seed once before creating
         # each Mechanism with GaussianDistort above
         assert np.allclose(run_results[0], [[-10.06333025,   2.4845505 ]])
-        if mode == 'Python':
-            assert np.allclose(ocm.feature_values, [[ 1.1576537,   0.60782117],
-                                                    [-0.03479106, -0.47666293],
-                                                    [-0.60836214,  0.1760381 ]])
+        if mode is pnl.ExecutionMode.Python:
+            assert np.allclose(ocm.state_feature_values, [[ 1.1576537,   0.60782117],
+                                                          [-0.03479106, -0.47666293],
+                                                          [-0.60836214,  0.1760381 ]])
 
     if benchmark.enabled:
-        benchmark(agent_comp.run, inputs=input_dict, bin_execute=mode)
+        benchmark(agent_comp.run, inputs=input_dict, execution_mode=mode)
