@@ -1,18 +1,18 @@
+import fractions
 import logging
 import numpy as np
 import psyneulink as pnl
 import pytest
 
-from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import DriftDiffusionIntegrator
-from psyneulink.core.components.functions.transferfunctions import Linear
+from psyneulink import _unit_registry
+from psyneulink.core.components.functions.stateful.integratorfunctions import DriftDiffusionIntegrator
+from psyneulink.core.components.functions.nonstateful.transferfunctions import Linear
 from psyneulink.core.components.mechanisms.processing.integratormechanism import IntegratorMechanism
 from psyneulink.core.components.mechanisms.processing.transfermechanism import TransferMechanism
 from psyneulink.core.components.projections.pathway.mappingprojection import MappingProjection
 from psyneulink.core.compositions.composition import Composition, EdgeType
-from psyneulink.core.globals.context import Context
 from psyneulink.core.globals.keywords import VALUE
-from psyneulink.core.scheduling.condition import AfterNCalls, AfterNPasses, AfterNTrials, AfterPass, All, AllHaveRun, Always, Any, AtPass, BeforeNCalls, BeforePass, \
-    EveryNCalls, EveryNPasses, JustRan, WhenFinished
+from psyneulink.core.scheduling.condition import AfterNCalls, AfterNPasses, AfterNTrials, AfterPass, All, AllHaveRun, Always, Any, AtPass, BeforeNCalls, BeforePass, EveryNCalls, EveryNPasses, JustRan, TimeInterval, WhenFinished
 from psyneulink.core.scheduling.scheduler import Scheduler
 from psyneulink.core.scheduling.time import TimeScale
 from psyneulink.library.components.mechanisms.processing.integrator.ddm import DDM
@@ -46,7 +46,7 @@ class TestScheduler:
         A = TransferMechanism(function=Linear(slope=5.0, intercept=2.0), name='scheduler-pytests-A')
         comp.add_node(A)
 
-        comp.scheduler.clock._increment_time(TimeScale.TRIAL)
+        comp.scheduler.get_clock(comp.scheduler.default_execution_id)._increment_time(TimeScale.TRIAL)
 
         eid = 'eid'
         eid1 = 'eid1'
@@ -54,7 +54,7 @@ class TestScheduler:
 
         assert comp.scheduler.clocks[eid].time.trial == 0
 
-        comp.scheduler.clock._increment_time(TimeScale.TRIAL)
+        comp.scheduler.get_clock(comp.scheduler.default_execution_id)._increment_time(TimeScale.TRIAL)
 
         assert comp.scheduler.clocks[eid].time.trial == 0
 
@@ -187,10 +187,10 @@ class TestScheduler:
         # Trial 1:
         # input = 2.0, termination condition = AllHaveRun
         # 1 pass (value = 2.0)
-        expected_results = [[np.array([[10.]]), np.array([[10.]])],
-                            [np.array([[2.]]), np.array([[1.]])],
-                            [np.array([[10.]]), np.array([[10.]])],
-                            [np.array([[2.]]), np.array([[1.]])]]
+        expected_results = [[np.array([10.]), np.array([10.])],
+                            [np.array([2.]), np.array([1.])],
+                            [np.array([10.]), np.array([10.])],
+                            [np.array([2.]), np.array([1.])]]
         assert np.allclose(expected_results, np.asfarray(C.results))
 
     def test_default_condition_1(self):
@@ -1168,7 +1168,7 @@ class TestTermination:
         assert output == pytest.helpers.setify_expected_output(expected_output)
 
         # reset the RUN because schedulers run TRIALs
-        sched.clock._increment_time(TimeScale.RUN)
+        sched.get_clock(sched.default_execution_id)._increment_time(TimeScale.RUN)
         sched._reset_counts_total(TimeScale.RUN, execution_id=sched.default_execution_id)
 
         output = list(sched.run())
@@ -1498,23 +1498,16 @@ class TestFeedback:
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
-    @pytest.mark.parametrize('mode', ['Python',
-                                      # 'LLVM' mode is not supported
-                                      # the comparison values and checks
-                                      # are not synced between binary
-                                      # and Python structures
-                                      pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                      pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                      pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                      pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                     ])
     @pytest.mark.parametrize('timescale, expected',
                              [(TimeScale.TIME_STEP, [[0.5], [0.4375]]),
                               (TimeScale.PASS, [[0.5], [0.4375]]),
                               (TimeScale.TRIAL, [[1.5], [0.4375]]),
                               (TimeScale.RUN, [[1.5], [0.4375]])],
                               ids=lambda x: x if isinstance(x, TimeScale) else "")
-    def test_time_termination_measures(self, mode, timescale, expected):
+    # 'LLVM' mode is not supported, because synchronization of compiler and
+    # python values during execution is not implemented.
+    @pytest.mark.usefixtures("comp_mode_no_llvm")
+    def test_time_termination_measures(self, comp_mode, timescale, expected):
         in_one_pass = timescale in {TimeScale.TIME_STEP, TimeScale.PASS}
         attention = pnl.TransferMechanism(name='Attention',
                                  integrator_mode=True,
@@ -1533,21 +1526,14 @@ class TestFeedback:
         comp.scheduler.add_condition(response, pnl.WhenFinished(attention))
         comp.scheduler.add_condition(counter, pnl.Always())
         inputs = {attention: [[0.5]], counter: [[2.0]]}
-        result = comp.run(inputs=inputs, bin_execute=mode)
-        if mode == 'Python':
+        result = comp.run(inputs=inputs, execution_mode=comp_mode)
+        if comp_mode is pnl.ExecutionMode.Python:
             assert attention.execution_count == 3
             assert counter.execution_count == 1 if in_one_pass else 3
             assert response.execution_count == 1
         assert np.allclose(result, expected)
 
     @pytest.mark.composition
-    @pytest.mark.parametrize("mode", ['Python',
-                                      # 'LLVM' mode is not supported, because synchronization of compiler and python values during execution is not implemented.
-                                      pytest.param('LLVMExec', marks=pytest.mark.llvm),
-                                      pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                      pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                      pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
-                                     ])
     @pytest.mark.parametrize("condition,scale,expected_result",
                              [(pnl.BeforeNCalls, TimeScale.TRIAL, [[.05, .05]]),
                               (pnl.BeforeNCalls, TimeScale.PASS, [[.05, .05]]),
@@ -1568,7 +1554,10 @@ class TestFeedback:
                               (pnl.AtTrial, None, [[0.05, 0.05]]),
                               #(pnl.Never), #TODO: Find a good test case for this!
                             ])
-    def test_scheduler_conditions(self, mode, condition, scale, expected_result):
+    # 'LLVM' mode is not supported, because synchronization of compiler and
+    # python values during execution is not implemented.
+    @pytest.mark.usefixtures("comp_mode_no_llvm")
+    def test_scheduler_conditions(self, comp_mode, condition, scale, expected_result):
         decisionMaker = pnl.DDM(
                         function=pnl.DriftDiffusionIntegrator(starting_point=0,
                                                               threshold=1,
@@ -1617,17 +1606,17 @@ class TestFeedback:
         elif condition is pnl.AtTrial:
             comp.scheduler.add_condition(response, condition(0))
 
-        result = comp.run([0.05], bin_execute=mode)
+        result = comp.run([0.05], execution_mode=comp_mode)
         #HACK: The result is an object dtype in Python mode for some reason?
-        if mode == 'Python':
+        if comp_mode is pnl.ExecutionMode.Python:
             result = np.asfarray(result[0])
         assert np.allclose(result, expected_result)
 
 
     @pytest.mark.composition
-    @pytest.mark.parametrize("mode", ['Python',
-                                      pytest.param('LLVMRun', marks=pytest.mark.llvm),
-                                      pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+    @pytest.mark.parametrize("mode", [pnl.ExecutionMode.Python,
+                                      pytest.param(pnl.ExecutionMode.LLVMRun, marks=pytest.mark.llvm),
+                                      pytest.param(pnl.ExecutionMode.PTXRun, marks=[pytest.mark.llvm, pytest.mark.cuda]),
                                      ])
     @pytest.mark.parametrize("condition,scale,expected_result",
                              [(pnl.AtTrial, None, [[[1.0]], [[2.0]]]),
@@ -1642,6 +1631,30 @@ class TestFeedback:
         comp.scheduler.termination_conds = {
             pnl.TimeScale.RUN: condition(2)
         }
-        r = comp.run(inputs=[1], num_trials=5, bin_execute=mode)
+        r = comp.run(inputs=[1], num_trials=5, execution_mode=mode)
         assert np.allclose(r, expected_result[-1])
         assert np.allclose(comp.results, expected_result)
+
+
+class TestAbsoluteTime:
+
+    @pytest.mark.parametrize(
+        'conditions, interval',
+        [
+            ({'A': TimeInterval(repeat=8), 'B': TimeInterval(repeat=4)}, fractions.Fraction(4, 3) * _unit_registry.ms),
+            ({'A': TimeInterval(repeat=1), 'B': TimeInterval(repeat=3)}, fractions.Fraction(1, 3) * _unit_registry.ms),
+            ({'A': Any(TimeInterval(repeat=2), TimeInterval(repeat=3))}, fractions.Fraction(1, 3) * _unit_registry.ms),
+            ({'A': TimeInterval(repeat=6), 'B': TimeInterval(repeat=3)}, 1 * _unit_registry.ms),
+            ({'A': TimeInterval(repeat=100 * _unit_registry.us), 'B': TimeInterval(repeat=2)}, fractions.Fraction(100, 3) * _unit_registry.us),
+            ({'A': Any(TimeInterval(repeat=1000 * _unit_registry.us), TimeInterval(repeat=2))}, fractions.Fraction(1, 3) * _unit_registry.ms),
+            ({'A': TimeInterval(repeat=1000 * _unit_registry.us), 'B': TimeInterval(repeat=2)}, fractions.Fraction(1, 3) * _unit_registry.ms),
+            ({'A': Any(TimeInterval(repeat=1000), TimeInterval(repeat=1500)), 'B': TimeInterval(repeat=2000)}, fractions.Fraction(500, 3) * _unit_registry.ms),
+        ]
+    )
+    def test_absolute_interval_linear(self, three_node_linear_composition, conditions, interval):
+        [A, B, C], comp = three_node_linear_composition
+
+        for node in conditions:
+            comp.scheduler.add_condition(eval(node), conditions[node])
+
+        assert comp.scheduler._get_absolute_consideration_set_execution_unit() == interval
